@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,46 +17,52 @@
  */
 
 #include "threadexecutor.h"
+
+#include "config.h"
 #include "cppcheck.h"
 #include "cppcheckexecutor.h"
+#include "importproject.h"
+#include "settings.h"
+#include "suppressions.h"
+
+#include <algorithm>
+#include <cerrno>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
+#include <utility>
+
 #ifdef __SVR4  // Solaris
 #include <sys/loadavg.h>
 #endif
 #ifdef THREADING_MODEL_FORK
-#include <algorithm>
+#if defined(__linux__)
+#include <sys/prctl.h>
+#endif
 #include <sys/select.h>
 #include <sys/wait.h>
-#include <unistd.h>
 #include <fcntl.h>
-#include <cstdlib>
-#include <cstdio>
-#include <errno.h>
-#include <time.h>
-#include <cstring>
-#include <sstream>
+#include <unistd.h>
 #endif
 #ifdef THREADING_MODEL_WIN
 #include <process.h>
 #include <windows.h>
-#include <algorithm>
-#include <cstring>
-#include <errno.h>
 #endif
 
 // required for FD_ZERO
 using std::memset;
 
 ThreadExecutor::ThreadExecutor(const std::map<std::string, std::size_t> &files, Settings &settings, ErrorLogger &errorLogger)
-    : _files(files), _settings(settings), _errorLogger(errorLogger), _fileCount(0)
+    : mFiles(files), mSettings(settings), mErrorLogger(errorLogger), mFileCount(0)
+      // Not initialized mFileSync, mErrorSync, mReportSync
 {
 #if defined(THREADING_MODEL_FORK)
-    _wpipe = 0;
+    mWpipe = 0;
 #elif defined(THREADING_MODEL_WIN)
-    _processedFiles = 0;
-    _totalFiles = 0;
-    _processedSize = 0;
-    _totalFileSize = 0;
+    mProcessedFiles = 0;
+    mTotalFiles = 0;
+    mProcessedSize = 0;
+    mTotalFileSize = 0;
 #endif
 }
 
@@ -74,7 +80,7 @@ ThreadExecutor::~ThreadExecutor()
 
 void ThreadExecutor::addFileContent(const std::string &path, const std::string &content)
 {
-    _fileContents[ path ] = content;
+    mFileContents[ path ] = content;
 }
 
 int ThreadExecutor::handleRead(int rpipe, unsigned int &result)
@@ -88,44 +94,41 @@ int ThreadExecutor::handleRead(int rpipe, unsigned int &result)
     }
 
     if (type != REPORT_OUT && type != REPORT_ERROR && type != REPORT_INFO && type != CHILD_END) {
-        std::cerr << "#### You found a bug from cppcheck.\nThreadExecutor::handleRead error, type was:" << type << std::endl;
+        std::cerr << "#### ThreadExecutor::handleRead error, type was:" << type << std::endl;
         std::exit(0);
     }
 
     unsigned int len = 0;
     if (read(rpipe, &len, sizeof(len)) <= 0) {
-        std::cerr << "#### You found a bug from cppcheck.\nThreadExecutor::handleRead error, type was:" << type << std::endl;
+        std::cerr << "#### ThreadExecutor::handleRead error, type was:" << type << std::endl;
         std::exit(0);
     }
 
-    char *buf = new char[len];
-    if (read(rpipe, buf, len) <= 0) {
-        std::cerr << "#### You found a bug from cppcheck.\nThreadExecutor::handleRead error, type was:" << type << std::endl;
+    // Don't rely on incoming data being null-terminated.
+    // Allocate +1 element and null-terminate the buffer.
+    char *buf = new char[len + 1];
+    const ssize_t readIntoBuf = read(rpipe, buf, len);
+    if (readIntoBuf <= 0) {
+        std::cerr << "#### ThreadExecutor::handleRead error, type was:" << type << std::endl;
         std::exit(0);
     }
+    buf[readIntoBuf] = 0;
 
     if (type == REPORT_OUT) {
-        _errorLogger.reportOut(buf);
+        mErrorLogger.reportOut(buf);
     } else if (type == REPORT_ERROR || type == REPORT_INFO) {
         ErrorLogger::ErrorMessage msg;
         msg.deserialize(buf);
 
-        std::string file;
-        unsigned int line(0);
-        if (!msg._callStack.empty()) {
-            file = msg._callStack.back().getfile(false);
-            line = msg._callStack.back().line;
-        }
-
-        if (!_settings.nomsg.isSuppressed(msg._id, file, line)) {
+        if (!mSettings.nomsg.isSuppressed(msg.toSuppressionsErrorMessage())) {
             // Alert only about unique errors
-            std::string errmsg = msg.toString(_settings.verbose);
-            if (std::find(_errorList.begin(), _errorList.end(), errmsg) == _errorList.end()) {
-                _errorList.push_back(errmsg);
+            std::string errmsg = msg.toString(mSettings.verbose);
+            if (std::find(mErrorList.begin(), mErrorList.end(), errmsg) == mErrorList.end()) {
+                mErrorList.push_back(errmsg);
                 if (type == REPORT_ERROR)
-                    _errorLogger.reportErr(msg);
+                    mErrorLogger.reportErr(msg);
                 else
-                    _errorLogger.reportInfo(msg);
+                    mErrorLogger.reportInfo(msg);
             }
         }
     } else if (type == CHILD_END) {
@@ -146,7 +149,7 @@ bool ThreadExecutor::checkLoadAverage(size_t nchildren)
 #if defined(__CYGWIN__) || defined(__QNX__)  // getloadavg() is unsupported on Cygwin, Qnx.
     return true;
 #else
-    if (!nchildren || !_settings.loadAverage) {
+    if (!nchildren || !mSettings.loadAverage) {
         return true;
     }
 
@@ -154,7 +157,7 @@ bool ThreadExecutor::checkLoadAverage(size_t nchildren)
     if (getloadavg(&sample, 1) != 1) {
         // disable load average checking on getloadavg error
         return true;
-    } else if (sample < _settings.loadAverage) {
+    } else if (sample < mSettings.loadAverage) {
         return true;
     }
     return false;
@@ -163,11 +166,11 @@ bool ThreadExecutor::checkLoadAverage(size_t nchildren)
 
 unsigned int ThreadExecutor::check()
 {
-    _fileCount = 0;
+    mFileCount = 0;
     unsigned int result = 0;
 
     std::size_t totalfilesize = 0;
-    for (std::map<std::string, std::size_t>::const_iterator i = _files.begin(); i != _files.end(); ++i) {
+    for (std::map<std::string, std::size_t>::const_iterator i = mFiles.begin(); i != mFiles.end(); ++i) {
         totalfilesize += i->second;
     }
 
@@ -175,47 +178,53 @@ unsigned int ThreadExecutor::check()
     std::map<pid_t, std::string> childFile;
     std::map<int, std::string> pipeFile;
     std::size_t processedsize = 0;
-    std::map<std::string, std::size_t>::const_iterator i = _files.begin();
+    std::map<std::string, std::size_t>::const_iterator iFile = mFiles.begin();
+    std::list<ImportProject::FileSettings>::const_iterator iFileSettings = mSettings.project.fileSettings.begin();
     for (;;) {
         // Start a new child
         size_t nchildren = rpipes.size();
-        if (i != _files.end() && nchildren < _settings.jobs && checkLoadAverage(nchildren)) {
+        if ((iFile != mFiles.end() || iFileSettings != mSettings.project.fileSettings.end()) && nchildren < mSettings.jobs && checkLoadAverage(nchildren)) {
             int pipes[2];
             if (pipe(pipes) == -1) {
-                std::cerr << "pipe() failed: "<< std::strerror(errno) << std::endl;
+                std::cerr << "#### ThreadExecutor::check, pipe() failed: "<< std::strerror(errno) << std::endl;
                 std::exit(EXIT_FAILURE);
             }
 
             int flags = 0;
             if ((flags = fcntl(pipes[0], F_GETFL, 0)) < 0) {
-                std::cerr << "fcntl(F_GETFL) failed: "<< std::strerror(errno) << std::endl;
+                std::cerr << "#### ThreadExecutor::check, fcntl(F_GETFL) failed: "<< std::strerror(errno) << std::endl;
                 std::exit(EXIT_FAILURE);
             }
 
             if (fcntl(pipes[0], F_SETFL, flags | O_NONBLOCK) < 0) {
-                std::cerr << "fcntl(F_SETFL) failed: "<< std::strerror(errno) << std::endl;
+                std::cerr << "#### ThreadExecutor::check, fcntl(F_SETFL) failed: "<< std::strerror(errno) << std::endl;
                 std::exit(EXIT_FAILURE);
             }
 
             pid_t pid = fork();
             if (pid < 0) {
                 // Error
-                std::cerr << "Failed to create child process: "<< std::strerror(errno) << std::endl;
+                std::cerr << "#### ThreadExecutor::check, Failed to create child process: "<< std::strerror(errno) << std::endl;
                 std::exit(EXIT_FAILURE);
             } else if (pid == 0) {
+#if defined(__linux__)
+                prctl(PR_SET_PDEATHSIG, SIGHUP);
+#endif
                 close(pipes[0]);
-                _wpipe = pipes[1];
+                mWpipe = pipes[1];
 
                 CppCheck fileChecker(*this, false);
-                fileChecker.settings() = _settings;
+                fileChecker.settings() = mSettings;
                 unsigned int resultOfCheck = 0;
 
-                if (!_fileContents.empty() && _fileContents.find(i->first) != _fileContents.end()) {
+                if (iFileSettings != mSettings.project.fileSettings.end()) {
+                    resultOfCheck = fileChecker.check(*iFileSettings);
+                } else if (!mFileContents.empty() && mFileContents.find(iFile->first) != mFileContents.end()) {
                     // File content was given as a string
-                    resultOfCheck = fileChecker.check(i->first, _fileContents[ i->first ]);
+                    resultOfCheck = fileChecker.check(iFile->first, mFileContents[ iFile->first ]);
                 } else {
                     // Read file from a file
-                    resultOfCheck = fileChecker.check(i->first);
+                    resultOfCheck = fileChecker.check(iFile->first);
                 }
 
                 std::ostringstream oss;
@@ -226,10 +235,15 @@ unsigned int ThreadExecutor::check()
 
             close(pipes[1]);
             rpipes.push_back(pipes[0]);
-            childFile[pid] = i->first;
-            pipeFile[pipes[0]] = i->first;
-
-            ++i;
+            if (iFileSettings != mSettings.project.fileSettings.end()) {
+                childFile[pid] = iFileSettings->filename + ' ' + iFileSettings->cfg;
+                pipeFile[pipes[0]] = iFileSettings->filename + ' ' + iFileSettings->cfg;
+                ++iFileSettings;
+            } else {
+                childFile[pid] = iFile->first;
+                pipeFile[pipes[0]] = iFile->first;
+                ++iFile;
+            }
         } else if (!rpipes.empty()) {
             fd_set rfds;
             FD_ZERO(&rfds);
@@ -251,16 +265,16 @@ unsigned int ThreadExecutor::check()
                             if (p != pipeFile.end()) {
                                 std::string name = p->second;
                                 pipeFile.erase(p);
-                                std::map<std::string, std::size_t>::const_iterator fs = _files.find(name);
-                                if (fs != _files.end()) {
+                                std::map<std::string, std::size_t>::const_iterator fs = mFiles.find(name);
+                                if (fs != mFiles.end()) {
                                     size = fs->second;
                                 }
                             }
 
-                            _fileCount++;
+                            mFileCount++;
                             processedsize += size;
-                            if (!_settings.quiet)
-                                CppCheckExecutor::reportStatus(_fileCount, _files.size(), processedsize, totalfilesize);
+                            if (!mSettings.quiet)
+                                CppCheckExecutor::reportStatus(mFileCount, mFiles.size() + mSettings.project.fileSettings.size(), processedsize, totalfilesize);
 
                             close(*rp);
                             rp = rpipes.erase(rp);
@@ -286,15 +300,16 @@ unsigned int ThreadExecutor::check()
                     oss << "Internal error: Child process crashed with signal " << WTERMSIG(stat);
 
                     std::list<ErrorLogger::ErrorMessage::FileLocation> locations;
-                    locations.push_back(ErrorLogger::ErrorMessage::FileLocation(childname, 0));
+                    locations.emplace_back(childname, 0, 0);
                     const ErrorLogger::ErrorMessage errmsg(locations,
+                                                           emptyString,
                                                            Severity::error,
                                                            oss.str(),
                                                            "cppcheckError",
                                                            false);
 
-                    if (!_settings.nomsg.isSuppressed(errmsg._id, childname, 0))
-                        _errorLogger.reportErr(errmsg);
+                    if (!mSettings.nomsg.isSuppressed(errmsg.toSuppressionsErrorMessage()))
+                        mErrorLogger.reportErr(errmsg);
                 }
             }
         } else {
@@ -314,9 +329,9 @@ void ThreadExecutor::writeToPipe(PipeSignal type, const std::string &data)
     out[0] = static_cast<char>(type);
     std::memcpy(&(out[1]), &len, sizeof(len));
     std::memcpy(&(out[1+sizeof(len)]), data.c_str(), len);
-    if (write(_wpipe, out, len + 1 + sizeof(len)) <= 0) {
+    if (write(mWpipe, out, len + 1 + sizeof(len)) <= 0) {
         delete [] out;
-        out = 0;
+        out = nullptr;
         std::cerr << "#### ThreadExecutor::writeToPipe, Failed to write to pipe" << std::endl;
         std::exit(0);
     }
@@ -339,70 +354,76 @@ void ThreadExecutor::reportInfo(const ErrorLogger::ErrorMessage &msg)
     writeToPipe(REPORT_INFO, msg.serialize());
 }
 
+void ThreadExecutor::bughuntingReport(const std::string &str)
+{
+    writeToPipe(REPORT_VERIFICATION, str.c_str());
+}
+
 #elif defined(THREADING_MODEL_WIN)
 
 void ThreadExecutor::addFileContent(const std::string &path, const std::string &content)
 {
-    _fileContents[path] = content;
+    mFileContents[path] = content;
 }
 
 unsigned int ThreadExecutor::check()
 {
-    HANDLE *threadHandles = new HANDLE[_settings.jobs];
+    HANDLE *threadHandles = new HANDLE[mSettings.jobs];
 
-    _itNextFile = _files.begin();
+    mItNextFile = mFiles.begin();
+    mItNextFileSettings = mSettings.project.fileSettings.begin();
 
-    _processedFiles = 0;
-    _processedSize = 0;
-    _totalFiles = _files.size();
-    _totalFileSize = 0;
-    for (std::map<std::string, std::size_t>::const_iterator i = _files.begin(); i != _files.end(); ++i) {
-        _totalFileSize += i->second;
+    mProcessedFiles = 0;
+    mProcessedSize = 0;
+    mTotalFiles = mFiles.size() + mSettings.project.fileSettings.size();
+    mTotalFileSize = 0;
+    for (std::map<std::string, std::size_t>::const_iterator i = mFiles.begin(); i != mFiles.end(); ++i) {
+        mTotalFileSize += i->second;
     }
 
-    InitializeCriticalSection(&_fileSync);
-    InitializeCriticalSection(&_errorSync);
-    InitializeCriticalSection(&_reportSync);
+    InitializeCriticalSection(&mFileSync);
+    InitializeCriticalSection(&mErrorSync);
+    InitializeCriticalSection(&mReportSync);
 
-    for (unsigned int i = 0; i < _settings.jobs; ++i) {
+    for (unsigned int i = 0; i < mSettings.jobs; ++i) {
         threadHandles[i] = (HANDLE)_beginthreadex(nullptr, 0, threadProc, this, 0, nullptr);
         if (!threadHandles[i]) {
-            std::cerr << "#### .\nThreadExecutor::check error, errno :" << errno << std::endl;
+            std::cerr << "#### ThreadExecutor::check error, errno :" << errno << std::endl;
             exit(EXIT_FAILURE);
         }
     }
 
-    DWORD waitResult = WaitForMultipleObjects(_settings.jobs, threadHandles, TRUE, INFINITE);
+    const DWORD waitResult = WaitForMultipleObjects(mSettings.jobs, threadHandles, TRUE, INFINITE);
     if (waitResult != WAIT_OBJECT_0) {
         if (waitResult == WAIT_FAILED) {
-            std::cerr << "#### .\nThreadExecutor::check wait failed, result: " << waitResult << " error: " << GetLastError() << std::endl;
+            std::cerr << "#### ThreadExecutor::check wait failed, result: " << waitResult << " error: " << GetLastError() << std::endl;
             exit(EXIT_FAILURE);
         } else {
-            std::cerr << "#### .\nThreadExecutor::check wait failed, result: " << waitResult << std::endl;
+            std::cerr << "#### ThreadExecutor::check wait failed, result: " << waitResult << std::endl;
             exit(EXIT_FAILURE);
         }
     }
 
     unsigned int result = 0;
-    for (unsigned int i = 0; i < _settings.jobs; ++i) {
+    for (unsigned int i = 0; i < mSettings.jobs; ++i) {
         DWORD exitCode;
 
         if (!GetExitCodeThread(threadHandles[i], &exitCode)) {
-            std::cerr << "#### .\nThreadExecutor::check get exit code failed, error:" << GetLastError() << std::endl;
+            std::cerr << "#### ThreadExecutor::check get exit code failed, error:" << GetLastError() << std::endl;
             exit(EXIT_FAILURE);
         }
 
         result += exitCode;
 
         if (!CloseHandle(threadHandles[i])) {
-            std::cerr << "#### .\nThreadExecutor::check close handle failed, error:" << GetLastError() << std::endl;
+            std::cerr << "#### ThreadExecutor::check close handle failed, error:" << GetLastError() << std::endl;
             exit(EXIT_FAILURE);
         }
     }
 
-    DeleteCriticalSection(&_fileSync);
-    DeleteCriticalSection(&_errorSync);
-    DeleteCriticalSection(&_reportSync);
+    DeleteCriticalSection(&mFileSync);
+    DeleteCriticalSection(&mErrorSync);
+    DeleteCriticalSection(&mReportSync);
 
     delete[] threadHandles;
 
@@ -414,43 +435,54 @@ unsigned int __stdcall ThreadExecutor::threadProc(void *args)
     unsigned int result = 0;
 
     ThreadExecutor *threadExecutor = static_cast<ThreadExecutor*>(args);
-    std::map<std::string, std::size_t>::const_iterator &it = threadExecutor->_itNextFile;
+    std::map<std::string, std::size_t>::const_iterator &itFile = threadExecutor->mItNextFile;
+    std::list<ImportProject::FileSettings>::const_iterator &itFileSettings = threadExecutor->mItNextFileSettings;
 
     // guard static members of CppCheck against concurrent access
-    EnterCriticalSection(&threadExecutor->_fileSync);
+    EnterCriticalSection(&threadExecutor->mFileSync);
 
     CppCheck fileChecker(*threadExecutor, false);
-    fileChecker.settings() = threadExecutor->_settings;
+    fileChecker.settings() = threadExecutor->mSettings;
 
     for (;;) {
-        if (it == threadExecutor->_files.end()) {
-            LeaveCriticalSection(&threadExecutor->_fileSync);
+        if (itFile == threadExecutor->mFiles.end() && itFileSettings == threadExecutor->mSettings.project.fileSettings.end()) {
+            LeaveCriticalSection(&threadExecutor->mFileSync);
             break;
-
-        }
-        const std::string &file = it->first;
-        const std::size_t fileSize = it->second;
-        ++it;
-
-        LeaveCriticalSection(&threadExecutor->_fileSync);
-
-        std::map<std::string, std::string>::const_iterator fileContent = threadExecutor->_fileContents.find(file);
-        if (fileContent != threadExecutor->_fileContents.end()) {
-            // File content was given as a string
-            result += fileChecker.check(file, fileContent->second);
-        } else {
-            // Read file from a file
-            result += fileChecker.check(file);
         }
 
-        EnterCriticalSection(&threadExecutor->_fileSync);
+        std::size_t fileSize = 0;
+        if (itFile != threadExecutor->mFiles.end()) {
+            const std::string &file = itFile->first;
+            fileSize = itFile->second;
+            ++itFile;
 
-        threadExecutor->_processedSize += fileSize;
-        threadExecutor->_processedFiles++;
-        if (!threadExecutor->_settings.quiet) {
-            EnterCriticalSection(&threadExecutor->_reportSync);
-            CppCheckExecutor::reportStatus(threadExecutor->_processedFiles, threadExecutor->_totalFiles, threadExecutor->_processedSize, threadExecutor->_totalFileSize);
-            LeaveCriticalSection(&threadExecutor->_reportSync);
+            LeaveCriticalSection(&threadExecutor->mFileSync);
+
+            const std::map<std::string, std::string>::const_iterator fileContent = threadExecutor->mFileContents.find(file);
+            if (fileContent != threadExecutor->mFileContents.end()) {
+                // File content was given as a string
+                result += fileChecker.check(file, fileContent->second);
+            } else {
+                // Read file from a file
+                result += fileChecker.check(file);
+            }
+        } else { // file settings..
+            const ImportProject::FileSettings &fs = *itFileSettings;
+            ++itFileSettings;
+            LeaveCriticalSection(&threadExecutor->mFileSync);
+            result += fileChecker.check(fs);
+            if (threadExecutor->mSettings.clangTidy)
+                fileChecker.analyseClangTidy(fs);
+        }
+
+        EnterCriticalSection(&threadExecutor->mFileSync);
+
+        threadExecutor->mProcessedSize += fileSize;
+        threadExecutor->mProcessedFiles++;
+        if (!threadExecutor->mSettings.quiet) {
+            EnterCriticalSection(&threadExecutor->mReportSync);
+            CppCheckExecutor::reportStatus(threadExecutor->mProcessedFiles, threadExecutor->mTotalFiles, threadExecutor->mProcessedSize, threadExecutor->mTotalFileSize);
+            LeaveCriticalSection(&threadExecutor->mReportSync);
         }
     }
     return result;
@@ -458,58 +490,56 @@ unsigned int __stdcall ThreadExecutor::threadProc(void *args)
 
 void ThreadExecutor::reportOut(const std::string &outmsg)
 {
-    EnterCriticalSection(&_reportSync);
+    EnterCriticalSection(&mReportSync);
 
-    _errorLogger.reportOut(outmsg);
+    mErrorLogger.reportOut(outmsg);
 
-    LeaveCriticalSection(&_reportSync);
+    LeaveCriticalSection(&mReportSync);
 }
 void ThreadExecutor::reportErr(const ErrorLogger::ErrorMessage &msg)
 {
-    report(msg, REPORT_ERROR);
+    report(msg, MessageType::REPORT_ERROR);
 }
 
 void ThreadExecutor::reportInfo(const ErrorLogger::ErrorMessage &msg)
 {
-    report(msg, REPORT_INFO);
+
+}
+
+void ThreadExecutor::bughuntingReport(const std::string  &/*str*/)
+{
+    // TODO
 }
 
 void ThreadExecutor::report(const ErrorLogger::ErrorMessage &msg, MessageType msgType)
 {
-    std::string file;
-    unsigned int line(0);
-    if (!msg._callStack.empty()) {
-        file = msg._callStack.back().getfile(false);
-        line = msg._callStack.back().line;
-    }
-
-    if (_settings.nomsg.isSuppressed(msg._id, file, line))
+    if (mSettings.nomsg.isSuppressed(msg.toSuppressionsErrorMessage()))
         return;
 
     // Alert only about unique errors
     bool reportError = false;
-    std::string errmsg = msg.toString(_settings.verbose);
+    const std::string errmsg = msg.toString(mSettings.verbose);
 
-    EnterCriticalSection(&_errorSync);
-    if (std::find(_errorList.begin(), _errorList.end(), errmsg) == _errorList.end()) {
-        _errorList.push_back(errmsg);
+    EnterCriticalSection(&mErrorSync);
+    if (std::find(mErrorList.begin(), mErrorList.end(), errmsg) == mErrorList.end()) {
+        mErrorList.push_back(errmsg);
         reportError = true;
     }
-    LeaveCriticalSection(&_errorSync);
+    LeaveCriticalSection(&mErrorSync);
 
     if (reportError) {
-        EnterCriticalSection(&_reportSync);
+        EnterCriticalSection(&mReportSync);
 
         switch (msgType) {
-        case REPORT_ERROR:
-            _errorLogger.reportErr(msg);
+        case MessageType::REPORT_ERROR:
+            mErrorLogger.reportErr(msg);
             break;
-        case REPORT_INFO:
-            _errorLogger.reportInfo(msg);
+        case MessageType::REPORT_INFO:
+            mErrorLogger.reportInfo(msg);
             break;
         }
 
-        LeaveCriticalSection(&_reportSync);
+        LeaveCriticalSection(&mReportSync);
     }
 }
 
@@ -539,4 +569,8 @@ void ThreadExecutor::reportInfo(const ErrorLogger::ErrorMessage &/*msg*/)
 
 }
 
+void ThreadExecutor::bughuntingReport(const std::string &/*str*/)
+{
+}
+
 #endif
