@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,16 +19,21 @@
 #ifndef THREADEXECUTOR_H
 #define THREADEXECUTOR_H
 
-#include <map>
-#include <string>
-#include <list>
+#include "config.h"
 #include "errorlogger.h"
 
-#if (defined(__GNUC__) || defined(__sun)) && !defined(__MINGW32__)
+#include <cstddef>
+#include <list>
+#include <map>
+#include <string>
+
+#if (defined(__GNUC__) || defined(__sun)) && !defined(__MINGW32__) && !defined(__CYGWIN__)
 #define THREADING_MODEL_FORK
 #elif defined(_WIN32)
 #define THREADING_MODEL_WIN
 #include <windows.h>
+
+#include "importproject.h"
 #endif
 
 class Settings;
@@ -42,13 +47,16 @@ class Settings;
  */
 class ThreadExecutor : public ErrorLogger {
 public:
-    ThreadExecutor(const std::map<std::string, std::size_t> &files, Settings &settings, ErrorLogger &_errorLogger);
-    virtual ~ThreadExecutor();
+    ThreadExecutor(const std::map<std::string, std::size_t> &files, Settings &settings, ErrorLogger &errorLogger);
+    ThreadExecutor(const ThreadExecutor &) = delete;
+    ~ThreadExecutor() OVERRIDE;
+    void operator=(const ThreadExecutor &) = delete;
     unsigned int check();
 
-    virtual void reportOut(const std::string &outmsg);
-    virtual void reportErr(const ErrorLogger::ErrorMessage &msg);
-    virtual void reportInfo(const ErrorLogger::ErrorMessage &msg);
+    void reportOut(const std::string &outmsg) OVERRIDE;
+    void reportErr(const ErrorLogger::ErrorMessage &msg) OVERRIDE;
+    void reportInfo(const ErrorLogger::ErrorMessage &msg) OVERRIDE;
+    void bughuntingReport(const std::string &str) OVERRIDE;
 
     /**
      * @brief Add content to a file, to be used in unit testing.
@@ -60,17 +68,17 @@ public:
     void addFileContent(const std::string &path, const std::string &content);
 
 private:
-    const std::map<std::string, std::size_t> &_files;
-    Settings &_settings;
-    ErrorLogger &_errorLogger;
-    unsigned int _fileCount;
+    const std::map<std::string, std::size_t> &mFiles;
+    Settings &mSettings;
+    ErrorLogger &mErrorLogger;
+    unsigned int mFileCount;
 
 #if defined(THREADING_MODEL_FORK)
 
     /** @brief Key is file name, and value is the content of the file */
-    std::map<std::string, std::string> _fileContents;
+    std::map<std::string, std::string> mFileContents;
 private:
-    enum PipeSignal {REPORT_OUT='1',REPORT_ERROR='2', REPORT_INFO='3', CHILD_END='4'};
+    enum PipeSignal {REPORT_OUT='1',REPORT_ERROR='2', REPORT_INFO='3', REPORT_VERIFICATION='4', CHILD_END='5'};
 
     /**
      * Read from the pipe, parse and handle what ever is in there.
@@ -84,12 +92,12 @@ private:
      * Write end of status pipe, different for each child.
      * Not used in master process.
      */
-    std::list<std::string> _errorList;
-    int _wpipe;
+    std::list<std::string> mErrorList;
+    int mWpipe;
 
     /**
      * @brief Check load average condition
-     * @param nchildren - count of currently runned children
+     * @param nchildren - count of currently ran children
      * @return true - if new process can be started
      */
     bool checkLoadAverage(size_t nchildren);
@@ -105,20 +113,21 @@ public:
 #elif defined(THREADING_MODEL_WIN)
 
 private:
-    enum MessageType {REPORT_ERROR, REPORT_INFO};
+    enum class MessageType {REPORT_ERROR, REPORT_INFO};
 
-    std::map<std::string, std::string> _fileContents;
-    std::map<std::string, std::size_t>::const_iterator _itNextFile;
-    std::size_t _processedFiles;
-    std::size_t _totalFiles;
-    std::size_t _processedSize;
-    std::size_t _totalFileSize;
-    CRITICAL_SECTION _fileSync;
+    std::map<std::string, std::string> mFileContents;
+    std::map<std::string, std::size_t>::const_iterator mItNextFile;
+    std::list<ImportProject::FileSettings>::const_iterator mItNextFileSettings;
+    std::size_t mProcessedFiles;
+    std::size_t mTotalFiles;
+    std::size_t mProcessedSize;
+    std::size_t mTotalFileSize;
+    CRITICAL_SECTION mFileSync;
 
-    std::list<std::string> _errorList;
-    CRITICAL_SECTION _errorSync;
+    std::list<std::string> mErrorList;
+    CRITICAL_SECTION mErrorSync;
 
-    CRITICAL_SECTION _reportSync;
+    CRITICAL_SECTION mReportSync;
 
     void report(const ErrorLogger::ErrorMessage &msg, MessageType msgType);
 
@@ -140,13 +149,6 @@ public:
         return false;
     }
 #endif
-
-private:
-    /** disabled copy constructor */
-    ThreadExecutor(const ThreadExecutor &);
-
-    /** disabled assignment operator */
-    void operator=(const ThreadExecutor &);
 };
 
 /// @}
