@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2018 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,10 +41,10 @@ namespace {
 
 void CheckAssert::assertWithSideEffects()
 {
-    if (!_settings->isEnabled(Settings::WARNING))
+    if (!mSettings->isEnabled(Settings::WARNING))
         return;
 
-    for (const Token* tok = _tokenizer->list.front(); tok; tok = tok->next()) {
+    for (const Token* tok = mTokenizer->list.front(); tok; tok = tok->next()) {
         if (!Token::simpleMatch(tok, "assert ("))
             continue;
 
@@ -64,7 +64,7 @@ void CheckAssert::assertWithSideEffects()
             if (!scope) continue;
 
             for (const Token *tok2 = scope->bodyStart; tok2 != scope->bodyEnd; tok2 = tok2->next()) {
-                if (tok2->tokType() != Token::eAssignmentOp && tok2->tokType() != Token::eIncDecOp)
+                if (!tok2->isAssignmentOp() && tok2->tokType() != Token::eIncDecOp)
                     continue;
 
                 const Variable* var = tok2->previous()->variable();
@@ -111,7 +111,7 @@ void CheckAssert::assignmentInAssertError(const Token *tok, const std::string& v
                 "assignmentInAssert",
                 "$symbol:" + varname + "\n"
                 "Assert statement modifies '$symbol'.\n"
-                "Variable '$symbol' is modified insert assert statement. "
+                "Variable '$symbol' is modified inside assert statement. "
                 "Assert statements are removed from release builds so the code inside "
                 "assert statement is not executed. If the code is needed also in release "
                 "builds, this is a bug.", CWE398, false);
@@ -120,13 +120,16 @@ void CheckAssert::assignmentInAssertError(const Token *tok, const std::string& v
 // checks if side effects happen on the variable prior to tmp
 void CheckAssert::checkVariableAssignment(const Token* assignTok, const Scope *assertionScope)
 {
-    const Variable* prevVar = assignTok->previous()->variable();
-    if (!prevVar)
+    if (!assignTok->isAssignmentOp() && assignTok->tokType() != Token::eIncDecOp)
+        return;
+
+    const Variable* var = assignTok->astOperand1()->variable();
+    if (!var)
         return;
 
     // Variable declared in inner scope in assert => don't warn
-    if (assertionScope != prevVar->scope()) {
-        const Scope *s = prevVar->scope();
+    if (assertionScope != var->scope()) {
+        const Scope *s = var->scope();
         while (s && s != assertionScope)
             s = s->nestedIn;
         if (s == assertionScope)
@@ -135,12 +138,12 @@ void CheckAssert::checkVariableAssignment(const Token* assignTok, const Scope *a
 
     // assignment
     if (assignTok->isAssignmentOp() || assignTok->tokType() == Token::eIncDecOp) {
-        if (prevVar->isConst())
+        if (var->isConst()) {
             return;
-
-        assignmentInAssertError(assignTok, prevVar->name());
+        }
+        assignmentInAssertError(assignTok, var->name());
     }
-    // TODO: function calls on prevVar
+    // TODO: function calls on var
 }
 
 bool CheckAssert::inSameScope(const Token* returnTok, const Token* assignTok)
