@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,26 @@
 #define checkexceptionsafetyH
 //---------------------------------------------------------------------------
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
+#include "errorlogger.h"
+#include "token.h"
+#include "tokenize.h"
+
+#include <list>
+#include <string>
+
+class Settings;
+
+// CWE ID used:
+static const struct CWE CWE398(398U);   // Indicator of Poor Code Quality
+static const struct CWE CWE703(703U);   // Improper Check or Handling of Exceptional Conditions
+
 
 /// @addtogroup Checks
 /// @{
 
 
-
 /**
  * @brief %Check exception safety (exceptions shouldn't cause leaks nor corrupt data)
  *
@@ -48,8 +60,7 @@ public:
         : Check(myName(), tokenizer, settings, errorLogger) {
     }
 
-    /** Checks that uses the simplified token list */
-    void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
+    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) OVERRIDE {
         if (tokenizer->isC())
             return;
 
@@ -87,53 +98,51 @@ private:
                     "Class " + className + " is not safe, destructor throws exception\n"
                     "The class " + className + " is not safe because its destructor "
                     "throws an exception. If " + className + " is used and an exception "
-                    "is thrown that is caught in an outer scope the program will terminate.");
+                    "is thrown that is caught in an outer scope the program will terminate.", CWE398, false);
     }
 
     void deallocThrowError(const Token * const tok, const std::string &varname) {
         reportError(tok, Severity::warning, "exceptDeallocThrow", "Exception thrown in invalid state, '" +
-                    varname + "' points at deallocated memory.");
+                    varname + "' points at deallocated memory.", CWE398, false);
     }
 
     void rethrowCopyError(const Token * const tok, const std::string &varname) {
         reportError(tok, Severity::style, "exceptRethrowCopy",
                     "Throwing a copy of the caught exception instead of rethrowing the original exception.\n"
                     "Rethrowing an exception with 'throw " + varname + ";' creates an unnecessary copy of '" + varname + "'. "
-                    "To rethrow the caught exception without unnecessary copying or slicing, use a bare 'throw;'.");
+                    "To rethrow the caught exception without unnecessary copying or slicing, use a bare 'throw;'.", CWE398, false);
     }
 
     void catchExceptionByValueError(const Token *tok) {
         reportError(tok, Severity::style,
                     "catchExceptionByValue", "Exception should be caught by reference.\n"
                     "The exception is caught by value. It could be caught "
-                    "as a (const) reference which is usually recommended in C++.");
+                    "as a (const) reference which is usually recommended in C++.", CWE398, false);
     }
 
     void noexceptThrowError(const Token * const tok) {
-        reportError(tok, Severity::error, "throwInNoexceptFunction", "Exception thrown in function declared not to throw exceptions.");
+        reportError(tok, Severity::error, "throwInNoexceptFunction", "Exception thrown in function declared not to throw exceptions.", CWE398, false);
     }
 
     /** Missing exception specification */
     void unhandledExceptionSpecificationError(const Token * const tok1, const Token * const tok2, const std::string & funcname) {
-        std::string str1(tok1 ? tok1->str() : "foo");
-        std::list<const Token*> locationList;
-        locationList.push_back(tok1);
-        locationList.push_back(tok2);
+        const std::string str1(tok1 ? tok1->str() : "foo");
+        const std::list<const Token*> locationList = { tok1, tok2 };
         reportError(locationList, Severity::style, "unhandledExceptionSpecification",
                     "Unhandled exception specification when calling function " + str1 + "().\n"
                     "Unhandled exception specification when calling function " + str1 + "(). "
-                    "Either use a try/catch around the function call, or add a exception specification for " + funcname + "() also.", CWE(0U), true);
+                    "Either use a try/catch around the function call, or add a exception specification for " + funcname + "() also.", CWE703, true);
     }
 
     /** Generate all possible errors (for --errorlist) */
-    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
-        CheckExceptionSafety c(0, settings, errorLogger);
-        c.destructorsError(0, "Class");
-        c.deallocThrowError(0, "p");
-        c.rethrowCopyError(0, "varname");
-        c.catchExceptionByValueError(0);
-        c.noexceptThrowError(0);
-        c.unhandledExceptionSpecificationError(0, 0, "funcname");
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const OVERRIDE {
+        CheckExceptionSafety c(nullptr, settings, errorLogger);
+        c.destructorsError(nullptr, "Class");
+        c.deallocThrowError(nullptr, "p");
+        c.rethrowCopyError(nullptr, "varname");
+        c.catchExceptionByValueError(nullptr);
+        c.noexceptThrowError(nullptr);
+        c.unhandledExceptionSpecificationError(nullptr, nullptr, "funcname");
     }
 
     /** Short description of class (for --doc) */
@@ -142,7 +151,7 @@ private:
     }
 
     /** wiki formatted description of the class (for --doc) */
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "Checking exception safety\n"
                "- Throwing exceptions in destructors\n"
                "- Throwing exception during invalid state\n"
