@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,18 @@
  * - CheckMemoryLeakStructMember checks allocation/deallocation of structs and struct members
  */
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
+#include "errorlogger.h"
+#include "tokenize.h"
 
 #include <list>
 #include <string>
 
-class Scope;
 class Function;
+class Scope;
+class Settings;
+class Token;
 class Variable;
 
 /// @addtogroup Core
@@ -47,35 +51,25 @@ class Variable;
 
 /** @brief Base class for memory leaks checking */
 class CPPCHECKLIB CheckMemoryLeak {
-protected:
-    /** For access to the tokens */
-    const Tokenizer * const tokenizer;
-
 private:
+    /** For access to the tokens */
+    const Tokenizer * const mTokenizer_;
+
     /** ErrorLogger used to report errors */
-    ErrorLogger * const errorLogger;
+    ErrorLogger * const mErrorLogger_;
 
     /** Enabled standards */
-    const Settings * const settings1;
-
-    /** Disable the default constructors */
-    CheckMemoryLeak();
-
-    /** Disable the default constructors */
-    CheckMemoryLeak(const CheckMemoryLeak &);
-
-    /** disable assignment operator */
-    void operator=(const CheckMemoryLeak &);
+    const Settings * const mSettings_;
 
     /**
      * Report error. Similar with the function Check::reportError
-     * @param location the token where the error occurs
+     * @param tok the token where the error occurs
      * @param severity the severity of the bug
      * @param id type of message
      * @param msg text
      * @param cwe cwe number
      */
-    void reportErr(const Token *location, Severity::SeverityType severity, const std::string &id, const std::string &msg, const CWE &cwe) const;
+    void reportErr(const Token *tok, Severity::SeverityType severity, const std::string &id, const std::string &msg, const CWE &cwe) const;
 
     /**
      * Report error. Similar with the function Check::reportError
@@ -88,14 +82,18 @@ private:
     void reportErr(const std::list<const Token *> &callstack, Severity::SeverityType severity, const std::string &id, const std::string &msg, const CWE &cwe) const;
 
 public:
+    CheckMemoryLeak() = delete;
+    CheckMemoryLeak(const CheckMemoryLeak &) = delete;
+    void operator=(const CheckMemoryLeak &) = delete;
+
     CheckMemoryLeak(const Tokenizer *t, ErrorLogger *e, const Settings *s)
-        : tokenizer(t), errorLogger(e), settings1(s) {
+        : mTokenizer_(t), mErrorLogger_(e), mSettings_(s) {
     }
 
     /** @brief What type of allocation are used.. the "Many" means that several types of allocation and deallocation are used */
     enum AllocType { No, Malloc, New, NewArray, File, Fd, Pipe, OtherMem, OtherRes, Many };
 
-    void memoryLeak(const Token *tok, const std::string &varname, AllocType alloctype);
+    void memoryLeak(const Token *tok, const std::string &varname, AllocType alloctype) const;
 
     /**
      * @brief Get type of deallocation at given position
@@ -103,26 +101,23 @@ public:
      * @param varid variable id
      * @return type of deallocation
      */
-    AllocType getDeallocationType(const Token *tok, unsigned int varid) const;
+    AllocType getDeallocationType(const Token *tok, nonneg int varid) const;
 
     /**
      * @brief Get type of allocation at given position
      */
-    AllocType getAllocationType(const Token *tok2, unsigned int varid, std::list<const Function*> *callstack = NULL) const;
+    AllocType getAllocationType(const Token *tok2, nonneg int varid, std::list<const Function*> *callstack = nullptr) const;
 
     /**
      * @brief Get type of reallocation at given position
      */
-    static AllocType getReallocationType(const Token *tok2, unsigned int varid);
+    AllocType getReallocationType(const Token *tok2, nonneg int varid) const;
 
     /**
-     * @brief Is a typename the name of a class?
-     * @param tok type token
-     * @param varid variable id
-     * @return true if the type name is the name of a class
+     * Check if token reopens a standard stream
+     * @param tok token to check
      */
-    bool isclass(const Token *tok, unsigned int varid) const;
-
+    bool isReopenStandardStream(const Token *tok) const;
     /**
      * Report that there is a memory leak (new/malloc/etc)
      * @param tok token where memory is leaked
@@ -146,13 +141,13 @@ public:
     void deallocuseError(const Token *tok, const std::string &varname) const;
     void mismatchSizeError(const Token *tok, const std::string &sz) const;
     void mismatchAllocDealloc(const std::list<const Token *> &callstack, const std::string &varname) const;
-    void memleakUponReallocFailureError(const Token *tok, const std::string &varname) const;
+    void memleakUponReallocFailureError(const Token *tok, const std::string &reallocfunction, const std::string &varname) const;
 
     /** What type of allocated memory does the given function return? */
-    AllocType functionReturnType(const Function* func, std::list<const Function*> *callstack = NULL) const;
+    AllocType functionReturnType(const Function* func, std::list<const Function*> *callstack = nullptr) const;
 
     /** Function allocates pointed-to argument (a la asprintf)? */
-    const char *functionArgAlloc(const Function *func, unsigned int targetpar, AllocType &allocType) const;
+    const char *functionArgAlloc(const Function *func, nonneg int targetpar, AllocType &allocType) const;
 };
 
 /// @}
@@ -176,125 +171,41 @@ public:
 class CPPCHECKLIB CheckMemoryLeakInFunction : private Check, public CheckMemoryLeak {
 public:
     /** @brief This constructor is used when registering this class */
-    CheckMemoryLeakInFunction() : Check(myName()), CheckMemoryLeak(0, 0, 0), symbolDatabase(NULL) {
+    CheckMemoryLeakInFunction() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
     /** @brief This constructor is used when running checks */
-    CheckMemoryLeakInFunction(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
-        : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
-        // get the symbol database
-        if (tokenizr)
-            symbolDatabase = tokenizr->getSymbolDatabase();
-        else
-            symbolDatabase = 0;
+    CheckMemoryLeakInFunction(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger), CheckMemoryLeak(tokenizer, errorLogger, settings) {
     }
 
-    /** @brief run all simplified checks */
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
-        CheckMemoryLeakInFunction checkMemoryLeak(tokenizr, settings, errLog);
+    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) OVERRIDE {
+        CheckMemoryLeakInFunction checkMemoryLeak(tokenizer, settings, errorLogger);
         checkMemoryLeak.checkReallocUsage();
-        checkMemoryLeak.check();
     }
 
     /** @brief Unit testing : testing the white list */
     static bool test_white_list(const std::string &funcname, const Settings *settings, bool cpp);
 
-    /** @brief Perform checking */
-    void check();
-
     /**
      * Checking for a memory leak caused by improper realloc usage.
      */
     void checkReallocUsage();
 
-    /**
-     * Inspect a function call. the call_func and getcode are recursive
-     * @param tok          token where the function call occurs
-     * @param callstack    callstack
-     * @param varid        variable id to check
-     * @param alloctype    if memory is allocated, this indicates the type of allocation
-     * @param dealloctype  if memory is deallocated, this indicates the type of deallocation
-     * @param allocpar     if function allocates varid parameter
-     * @param sz           not used by call_func - see getcode
-     * @return These are the possible return values:
-     * - NULL : no significant code
-     * - "recursive" : recursive function
-     * - "alloc" : the function returns allocated memory
-     * - "dealloc" : the function deallocates the variable
-     * - "dealloc_"
-     * - "use" : the variable is used (unknown usage of the variable => the checking bails out)
-     * - "callfunc" : a function call with unknown side effects
-     * - "&use"
-     */
-    const char * call_func(const Token *tok, std::list<const Token *> callstack, const unsigned int varid, AllocType &alloctype, AllocType &dealloctype, bool &allocpar, unsigned int sz);
-
-    /**
-     * Extract a new tokens list that is easier to parse than the "_tokenizer->tokens()", the
-     * extracted tokens list describes how the given variable is used.
-     * The getcode and call_func are recursive
-     * @param tok start parse token
-     * @param callstack callstack
-     * @param varid variable id
-     * @param alloctype keep track of what type of allocation is used
-     * @param dealloctype keeps track of what type of deallocation is used
-     * @param classmember should be set if the inspected function is a class member
-     * @param sz size of type, used to check for mismatching size of allocation. for example "int *a;" => the sz is "sizeof(int)"
-     * @return Newly allocated token array. Caller needs to release reserved
-     * memory by calling TokenList::deleteTokens(returnValue);
-     * Returned tokens:
-     * - "alloc" : the variable is allocated
-     * - "assign" : the variable is assigned a new value
-     * - "break" : corresponds to "break"
-     * - "callfunc" : a function call with unknown side effects
-     * - "continue" : corresponds to "continue"
-     * - "dealloc" : the variable is deallocated
-     * - "goto" : corresponds to a "goto"
-     * - "if" : there is an "if"
-     * - "if(var)" : corresponds with "if ( var != 0 )"
-     * - "if(!var)" : corresponds with "if ( var == 0 )"
-     * - "ifv" : the variable is used in some way in a "if"
-     * - "loop" : corresponds to either a "for" or a "while"
-     * - "realloc" : the variable is reallocated
-     * - "return" : corresponds to a "return"
-     * - "use" : unknown usage -> bail out checking of this execution path
-     * - "&use" : the address of the variable is taken
-     * - "::use" : calling member function of class
-     * - "use_" : content of variable is accessed (used to warn access after dealloc)
-     */
-    Token *getcode(const Token *tok, std::list<const Token *> callstack, const unsigned int varid, AllocType &alloctype, AllocType &dealloctype, bool classmember, unsigned int sz);
-
-    /**
-     * Simplify code e.g. by replacing empty "{ }" with ";"
-     * @param tok first token. The tokens list can be modified.
-     */
-    void simplifycode(Token *tok) const;
-
-    static const Token *findleak(const Token *tokens);
-
-    /**
-     * Checking the variable varname
-     * @param startTok start token
-     * @param varname name of variable (for error messages)
-     * @param varid variable id
-     * @param classmember is the scope inside a class member function
-     * @param sz size of type.. if the variable is a "int *" then sz should be "sizeof(int)"
-     */
-    void checkScope(const Token *startTok, const std::string &varname, unsigned int varid, bool classmember, unsigned int sz);
-
 private:
     /** Report all possible errors (for the --errorlist) */
-    void getErrorMessages(ErrorLogger *e, const Settings *settings) const {
-        CheckMemoryLeakInFunction c(0, settings, e);
+    void getErrorMessages(ErrorLogger *e, const Settings *settings) const OVERRIDE {
+        CheckMemoryLeakInFunction c(nullptr, settings, e);
 
-        c.memleakError(0, "varname");
-        c.resourceLeakError(0, "varname");
+        c.memleakError(nullptr, "varname");
+        c.resourceLeakError(nullptr, "varname");
 
-        c.deallocDeallocError(0, "varname");
-        c.deallocuseError(0, "varname");
-        c.mismatchSizeError(0, "sz");
-        std::list<const Token *> callstack;
+        c.deallocDeallocError(nullptr, "varname");
+        c.deallocuseError(nullptr, "varname");
+        c.mismatchSizeError(nullptr, "sz");
+        const std::list<const Token *> callstack;
         c.mismatchAllocDealloc(callstack, "varname");
-        c.memleakUponReallocFailureError(0, "varname");
+        c.memleakUponReallocFailureError(nullptr, "realloc", "varname");
     }
 
     /**
@@ -309,11 +220,9 @@ private:
      * Get class information (--doc)
      * @return Wiki formatted information about this class
      */
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "Is there any allocated memory when a function goes out of scope\n";
     }
-
-    const SymbolDatabase *symbolDatabase;
 };
 
 
@@ -324,14 +233,14 @@ private:
 
 class CPPCHECKLIB CheckMemoryLeakInClass : private Check, private CheckMemoryLeak {
 public:
-    CheckMemoryLeakInClass() : Check(myName()), CheckMemoryLeak(0, 0, 0) {
+    CheckMemoryLeakInClass() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
-    CheckMemoryLeakInClass(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
-        : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
+    CheckMemoryLeakInClass(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger), CheckMemoryLeak(tokenizer, errorLogger, settings) {
     }
 
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
+    void runChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) OVERRIDE {
         if (!tokenizr->isCPP())
             return;
 
@@ -350,17 +259,17 @@ private:
 
     void unsafeClassError(const Token *tok, const std::string &classname, const std::string &varname);
 
-    void getErrorMessages(ErrorLogger *e, const Settings *settings) const {
-        CheckMemoryLeakInClass c(0, settings, e);
-        c.publicAllocationError(0, "varname");
-        c.unsafeClassError(0, "class", "class::varname");
+    void getErrorMessages(ErrorLogger *e, const Settings *settings) const OVERRIDE {
+        CheckMemoryLeakInClass c(nullptr, settings, e);
+        c.publicAllocationError(nullptr, "varname");
+        c.unsafeClassError(nullptr, "class", "class::varname");
     }
 
     static std::string myName() {
         return "Memory leaks (class variables)";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "If the constructor allocate memory then the destructor must deallocate it.\n";
     }
 };
@@ -371,15 +280,15 @@ private:
 
 class CPPCHECKLIB CheckMemoryLeakStructMember : private Check, private CheckMemoryLeak {
 public:
-    CheckMemoryLeakStructMember() : Check(myName()), CheckMemoryLeak(0, 0, 0) {
+    CheckMemoryLeakStructMember() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
-    CheckMemoryLeakStructMember(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
-        : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
+    CheckMemoryLeakStructMember(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger), CheckMemoryLeak(tokenizer, errorLogger, settings) {
     }
 
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
-        CheckMemoryLeakStructMember checkMemoryLeak(tokenizr, settings, errLog);
+    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) OVERRIDE {
+        CheckMemoryLeakStructMember checkMemoryLeak(tokenizer, settings, errorLogger);
         checkMemoryLeak.check();
     }
 
@@ -392,14 +301,14 @@ private:
 
     void checkStructVariable(const Variable * const variable);
 
-    void getErrorMessages(ErrorLogger * /*errorLogger*/, const Settings * /*settings*/) const {
+    void getErrorMessages(ErrorLogger * /*errorLogger*/, const Settings * /*settings*/) const OVERRIDE {
     }
 
     static std::string myName() {
         return "Memory leaks (struct members)";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "Don't forget to deallocate struct members\n";
     }
 };
@@ -410,21 +319,28 @@ private:
 
 class CPPCHECKLIB CheckMemoryLeakNoVar : private Check, private CheckMemoryLeak {
 public:
-    CheckMemoryLeakNoVar() : Check(myName()), CheckMemoryLeak(0, 0, 0) {
+    CheckMemoryLeakNoVar() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
-    CheckMemoryLeakNoVar(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
-        : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
+    CheckMemoryLeakNoVar(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger), CheckMemoryLeak(tokenizer, errorLogger, settings) {
     }
 
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
-        CheckMemoryLeakNoVar checkMemoryLeak(tokenizr, settings, errLog);
+    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) OVERRIDE {
+        CheckMemoryLeakNoVar checkMemoryLeak(tokenizer, settings, errorLogger);
         checkMemoryLeak.check();
     }
 
     void check();
 
 private:
+    /**
+     * @brief %Check if an input argument to a function is the return value of an allocation function
+     * like malloc(), and the function does not release it.
+     * @param scope     The scope of the function to check.
+     */
+    void checkForUnreleasedInputArgument(const Scope *scope);
+
     /**
      * @brief %Check if a call to an allocation function like malloc() is made and its return value is not assigned.
      * @param scope     The scope of the function to check.
@@ -441,19 +357,19 @@ private:
     void returnValueNotUsedError(const Token* tok, const std::string &alloc);
     void unsafeArgAllocError(const Token *tok, const std::string &funcName, const std::string &ptrType, const std::string &objType);
 
-    void getErrorMessages(ErrorLogger *e, const Settings *settings) const {
-        CheckMemoryLeakNoVar c(0, settings, e);
+    void getErrorMessages(ErrorLogger *e, const Settings *settings) const OVERRIDE {
+        CheckMemoryLeakNoVar c(nullptr, settings, e);
 
-        c.functionCallLeak(0, "funcName", "funcName");
-        c.returnValueNotUsedError(0, "funcName");
-        c.unsafeArgAllocError(0, "funcName", "shared_ptr", "int");
+        c.functionCallLeak(nullptr, "funcName", "funcName");
+        c.returnValueNotUsedError(nullptr, "funcName");
+        c.unsafeArgAllocError(nullptr, "funcName", "shared_ptr", "int");
     }
 
     static std::string myName() {
         return "Memory leaks (address not taken)";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "Not taking the address to allocated memory\n";
     }
 };
