@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +22,22 @@
 #define checkotherH
 //---------------------------------------------------------------------------
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
+#include "errorlogger.h"
+#include "utils.h"
 
-class Function;
+#include <cstddef>
+#include <string>
+#include <vector>
+
+namespace ValueFlow {
+    class Value;
+}
+
+class Settings;
+class Token;
+class Tokenizer;
 class Variable;
 
 /// @addtogroup Checks
@@ -46,7 +58,7 @@ public:
     }
 
     /** @brief Run checks against the normal token list */
-    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
+    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) OVERRIDE {
         CheckOther checkOther(tokenizer, settings, errorLogger);
 
         // Checks
@@ -72,29 +84,22 @@ public:
         checkOther.checkInterlockedDecrement();
         checkOther.checkUnusedLabel();
         checkOther.checkEvaluationOrder();
-    }
-
-    /** @brief Run checks against the simplified token list */
-    void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
-        CheckOther checkOther(tokenizer, settings, errorLogger);
-
-        // Checks
-        checkOther.clarifyCalculation();
-        checkOther.clarifyStatement();
-        checkOther.checkConstantFunctionParameter();
+        checkOther.checkFuncArgNamesDifferent();
+        checkOther.checkShadowVariables();
+        checkOther.checkKnownArgument();
+        checkOther.checkComparePointers();
         checkOther.checkIncompleteStatement();
-        checkOther.checkCastIntToCharAndBack();
-
-        checkOther.checkMisusedScopedObject();
-        checkOther.checkMemsetZeroBytes();
-        checkOther.checkMemsetInvalid2ndParam();
-        checkOther.checkSwitchCaseFallThrough();
         checkOther.checkPipeParameterSize();
-
-        checkOther.checkInvalidFree();
         checkOther.checkRedundantCopy();
-        checkOther.checkSuspiciousEqualityComparison();
+        checkOther.clarifyCalculation();
+        checkOther.checkPassByReference();
+        checkOther.checkConstVariable();
         checkOther.checkComparisonFunctionIsAlwaysTrueOrFalse();
+        checkOther.checkInvalidFree();
+        checkOther.clarifyStatement();
+        checkOther.checkCastIntToCharAndBack();
+        checkOther.checkMisusedScopedObject();
+        checkOther.checkAccessOfMovedVariable();
     }
 
     /** @brief Clarify calculation for ".. a * b ? .." */
@@ -116,8 +121,10 @@ public:
     /** @brief %Check for comma separated statements in return */
     void checkCommaSeparatedReturn();
 
-    /** @brief %Check for constant function parameter */
-    void checkConstantFunctionParameter();
+    /** @brief %Check for function parameters that should be passed by reference */
+    void checkPassByReference();
+
+    void checkConstVariable();
 
     /** @brief Using char variable as array index / as operand in bit operation */
     void checkCharVariable();
@@ -140,21 +147,9 @@ public:
     /** @brief %Check for code like 'case A||B:'*/
     void checkSuspiciousCaseInSwitch();
 
-    /** @brief %Check for code like 'case A||B:'*/
-    void checkSuspiciousEqualityComparison();
-
-    /** @brief %Check for switch case fall through without comment */
-    void checkSwitchCaseFallThrough();
-
     /** @brief %Check for objects that are destroyed immediately */
     void checkMisusedScopedObject();
 
-    /** @brief %Check for filling zero bytes with memset() */
-    void checkMemsetZeroBytes();
-
-    /** @brief %Check for invalid 2nd parameter of memset() */
-    void checkMemsetInvalid2ndParam();
-
     /** @brief %Check for suspicious code where if and else branch are the same (e.g "if (a) b = true; else b = true;") */
     void checkDuplicateBranch();
 
@@ -172,7 +167,7 @@ public:
 
     /** @brief %Check for free() operations on invalid memory locations */
     void checkInvalidFree();
-    void invalidFreeError(const Token *tok, bool inconclusive);
+    void invalidFreeError(const Token *tok, const std::string &allocation, bool inconclusive);
 
     /** @brief %Check for code creating redundant copies */
     void checkRedundantCopy();
@@ -207,47 +202,59 @@ public:
     /** @brief %Check for expression that depends on order of evaluation of side effects */
     void checkEvaluationOrder();
 
+    /** @brief %Check for access of moved or forwarded variable */
+    void checkAccessOfMovedVariable();
+
+    /** @brief %Check if function declaration and definition argument names different */
+    void checkFuncArgNamesDifferent();
+
+    /** @brief %Check for shadow variables. Less noisy than gcc/clang -Wshadow. */
+    void checkShadowVariables();
+
+    void checkKnownArgument();
+
+    void checkComparePointers();
+
 private:
     // Error messages..
-    void checkComparisonFunctionIsAlwaysTrueOrFalseError(const Token* tok, const std::string &strFunctionName, const std::string &varName, const bool result);
+    void checkComparisonFunctionIsAlwaysTrueOrFalseError(const Token* tok, const std::string &functionName, const std::string &varName, const bool result);
     void checkCastIntToCharAndBackError(const Token *tok, const std::string &strFunctionName);
     void checkPipeParameterSizeError(const Token *tok, const std::string &strVarName, const std::string &strDim);
     void clarifyCalculationError(const Token *tok, const std::string &op);
     void clarifyStatementError(const Token* tok);
     void cstyleCastError(const Token *tok);
-    void invalidPointerCastError(const Token* tok, const std::string& from, const std::string& to, bool inconclusive);
-    void passedByValueError(const Token *tok, const std::string &parname);
-    void constStatementError(const Token *tok, const std::string &type);
+    void invalidPointerCastError(const Token* tok, const std::string& from, const std::string& to, bool inconclusive, bool toIsInt);
+    void passedByValueError(const Token *tok, const std::string &parname, bool inconclusive);
+    void constVariableError(const Variable *var);
+    void constStatementError(const Token *tok, const std::string &type, bool inconclusive);
     void signedCharArrayIndexError(const Token *tok);
     void unknownSignCharArrayIndexError(const Token *tok);
     void charBitOpError(const Token *tok);
     void variableScopeError(const Token *tok, const std::string &varname);
-    void zerodivError(const Token *tok, bool inconclusive);
-    void zerodivcondError(const Token *tokcond, const Token *tokdiv, bool inconclusive);
+    void zerodivError(const Token *tok, const ValueFlow::Value *value);
     void nanInArithmeticExpressionError(const Token *tok);
     void redundantAssignmentError(const Token *tok1, const Token* tok2, const std::string& var, bool inconclusive);
+    void redundantInitializationError(const Token *tok1, const Token* tok2, const std::string& var, bool inconclusive);
     void redundantAssignmentInSwitchError(const Token *tok1, const Token *tok2, const std::string &var);
     void redundantCopyError(const Token *tok1, const Token* tok2, const std::string& var);
     void redundantCopyInSwitchError(const Token *tok1, const Token* tok2, const std::string &var);
     void redundantBitwiseOperationInSwitchError(const Token *tok, const std::string &varname);
-    void switchCaseFallThrough(const Token *tok);
     void suspiciousCaseInSwitchError(const Token* tok, const std::string& operatorString);
-    void suspiciousEqualityComparisonError(const Token* tok);
     void selfAssignmentError(const Token *tok, const std::string &varname);
     void misusedScopeObjectError(const Token *tok, const std::string &varname);
-    void memsetZeroBytesError(const Token *tok);
-    void memsetFloatError(const Token *tok, const std::string &var_value);
-    void memsetValueOutOfRangeError(const Token *tok, const std::string &value);
-    void duplicateBranchError(const Token *tok1, const Token *tok2);
-    void duplicateExpressionError(const Token *tok1, const Token *tok2, const std::string &op);
-    void duplicateExpressionTernaryError(const Token *tok);
+    void duplicateBranchError(const Token *tok1, const Token *tok2, ErrorPath errors);
+    void duplicateAssignExpressionError(const Token *tok1, const Token *tok2, bool inconclusive);
+    void oppositeExpressionError(const Token *opTok, ErrorPath errors);
+    void duplicateExpressionError(const Token *tok1, const Token *tok2, const Token *opTok, ErrorPath errors);
+    void duplicateValueTernaryError(const Token *tok);
+    void duplicateExpressionTernaryError(const Token *tok, ErrorPath errors);
     void duplicateBreakError(const Token *tok, bool inconclusive);
     void unreachableCodeError(const Token* tok, bool inconclusive);
-    void unsignedLessThanZeroError(const Token *tok, const std::string &varname, bool inconclusive);
-    void pointerLessThanZeroError(const Token *tok, bool inconclusive);
-    void unsignedPositiveError(const Token *tok, const std::string &varname, bool inconclusive);
-    void pointerPositiveError(const Token *tok, bool inconclusive);
-    void SuspiciousSemicolonError(const Token *tok);
+    void unsignedLessThanZeroError(const Token *tok, const ValueFlow::Value *v, const std::string &varname);
+    void pointerLessThanZeroError(const Token *tok, const ValueFlow::Value *v);
+    void unsignedPositiveError(const Token *tok, const ValueFlow::Value *v, const std::string &varname);
+    void pointerPositiveError(const Token *tok, const ValueFlow::Value *v);
+    void suspiciousSemicolonError(const Token *tok);
     void negativeBitwiseShiftError(const Token *tok, int op);
     void redundantCopyError(const Token *tok, const std::string &varname);
     void incompleteArrayFillError(const Token* tok, const std::string& buffer, const std::string& function, bool boolean);
@@ -257,55 +264,63 @@ private:
     void raceAfterInterlockedDecrementError(const Token* tok);
     void unusedLabelError(const Token* tok, bool inSwitch);
     void unknownEvaluationOrder(const Token* tok);
+    static bool isMovedParameterAllowedForInconclusiveFunction(const Token * tok);
+    void accessMovedError(const Token *tok, const std::string &varname, const ValueFlow::Value *value, bool inconclusive);
+    void funcArgNamesDifferent(const std::string & functionName, nonneg int index, const Token* declaration, const Token* definition);
+    void funcArgOrderDifferent(const std::string & functionName, const Token * declaration, const Token * definition, const std::vector<const Token*> & declarations, const std::vector<const Token*> & definitions);
+    void shadowError(const Token *var, const Token *shadowed, std::string type);
+    void knownArgumentError(const Token *tok, const Token *ftok, const ValueFlow::Value *value);
+    void comparePointersError(const Token *tok, const ValueFlow::Value *v1, const ValueFlow::Value *v2);
 
-    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const OVERRIDE {
         CheckOther c(nullptr, settings, errorLogger);
 
+        ErrorPath errorPath;
+
         // error
-        c.zerodivError(nullptr,  false);
-        c.zerodivcondError(nullptr, 0,false);
+        c.zerodivError(nullptr, nullptr);
         c.misusedScopeObjectError(nullptr, "varname");
-        c.invalidPointerCastError(nullptr,  "float", "double", false);
+        c.invalidPointerCastError(nullptr,  "float *", "double *", false, false);
         c.negativeBitwiseShiftError(nullptr, 1);
+        c.negativeBitwiseShiftError(nullptr, 2);
         c.checkPipeParameterSizeError(nullptr,  "varname", "dimension");
         c.raceAfterInterlockedDecrementError(nullptr);
+        c.invalidFreeError(nullptr, "malloc", false);
 
         //performance
         c.redundantCopyError(nullptr,  "varname");
-        c.redundantCopyError(nullptr,  0, "var");
-        c.redundantAssignmentError(nullptr,  0, "var", false);
+        c.redundantCopyError(nullptr, nullptr, "var");
 
         // style/warning
         c.checkComparisonFunctionIsAlwaysTrueOrFalseError(nullptr, "isless","varName",false);
         c.checkCastIntToCharAndBackError(nullptr, "func_name");
         c.cstyleCastError(nullptr);
-        c.passedByValueError(nullptr,  "parametername");
-        c.constStatementError(nullptr,  "type");
+        c.passedByValueError(nullptr, "parametername", false);
+        c.constVariableError(nullptr);
+        c.constStatementError(nullptr, "type", false);
         c.signedCharArrayIndexError(nullptr);
         c.unknownSignCharArrayIndexError(nullptr);
         c.charBitOpError(nullptr);
         c.variableScopeError(nullptr,  "varname");
-        c.redundantAssignmentInSwitchError(nullptr,  0, "var");
-        c.redundantCopyInSwitchError(nullptr,  0, "var");
-        c.switchCaseFallThrough(nullptr);
+        c.redundantAssignmentInSwitchError(nullptr, nullptr, "var");
+        c.redundantCopyInSwitchError(nullptr, nullptr, "var");
         c.suspiciousCaseInSwitchError(nullptr,  "||");
-        c.suspiciousEqualityComparisonError(nullptr);
         c.selfAssignmentError(nullptr,  "varname");
-        c.memsetZeroBytesError(nullptr);
-        c.memsetFloatError(nullptr,  "varname");
-        c.memsetValueOutOfRangeError(nullptr,  "varname");
         c.clarifyCalculationError(nullptr,  "+");
         c.clarifyStatementError(nullptr);
-        c.duplicateBranchError(nullptr,  0);
-        c.duplicateExpressionError(nullptr,  0, "&&");
-        c.duplicateExpressionTernaryError(nullptr);
+        c.duplicateBranchError(nullptr, nullptr, errorPath);
+        c.duplicateAssignExpressionError(nullptr, nullptr, true);
+        c.oppositeExpressionError(nullptr, errorPath);
+        c.duplicateExpressionError(nullptr, nullptr, nullptr, errorPath);
+        c.duplicateValueTernaryError(nullptr);
+        c.duplicateExpressionTernaryError(nullptr, errorPath);
         c.duplicateBreakError(nullptr,  false);
         c.unreachableCodeError(nullptr,  false);
-        c.unsignedLessThanZeroError(nullptr,  "varname", false);
-        c.unsignedPositiveError(nullptr,  "varname", false);
-        c.pointerLessThanZeroError(nullptr,  false);
-        c.pointerPositiveError(nullptr,  false);
-        c.SuspiciousSemicolonError(nullptr);
+        c.unsignedLessThanZeroError(nullptr, nullptr, "varname");
+        c.unsignedPositiveError(nullptr, nullptr, "varname");
+        c.pointerLessThanZeroError(nullptr, nullptr);
+        c.pointerPositiveError(nullptr, nullptr);
+        c.suspiciousSemicolonError(nullptr);
         c.incompleteArrayFillError(nullptr,  "buffer", "memset", false);
         c.varFuncNullUBError(nullptr);
         c.nanInArithmeticExpressionError(nullptr);
@@ -314,13 +329,26 @@ private:
         c.unusedLabelError(nullptr,  true);
         c.unusedLabelError(nullptr,  false);
         c.unknownEvaluationOrder(nullptr);
+        c.accessMovedError(nullptr, "v", nullptr, false);
+        c.funcArgNamesDifferent("function", 1, nullptr, nullptr);
+        c.redundantBitwiseOperationInSwitchError(nullptr, "varname");
+        c.shadowError(nullptr, nullptr, "variable");
+        c.shadowError(nullptr, nullptr, "function");
+        c.shadowError(nullptr, nullptr, "argument");
+        c.knownArgumentError(nullptr, nullptr, nullptr);
+        c.comparePointersError(nullptr, nullptr, nullptr);
+        c.redundantAssignmentError(nullptr, nullptr, "var", false);
+        c.redundantInitializationError(nullptr, nullptr, "var", false);
+
+        const std::vector<const Token *> nullvec;
+        c.funcArgOrderDifferent("function", nullptr, nullptr, nullvec, nullvec);
     }
 
     static std::string myName() {
         return "Other";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "Other checks\n"
 
                // error
@@ -336,20 +364,19 @@ private:
 
                // warning
                "- either division by zero or useless condition\n"
-               "- memset() with a value out of range as the 2nd parameter\n"
+               "- access of moved or forwarded variable.\n"
 
                // performance
                "- redundant data copying for const variable\n"
                "- subsequent assignment or copying to a variable or buffer\n"
+               "- passing parameter by value\n"
 
                // portability
-               "- memset() with a float as the 2nd parameter\n"
                "- Passing NULL pointer to function with variable number of arguments leads to UB.\n"
 
                // style
                "- C-style pointer cast in C++ code\n"
                "- casting between incompatible pointer types\n"
-               "- passing parameter by value\n"
                "- [Incomplete statement](IncompleteStatement)\n"
                "- [check how signed char variables are used](CharVar)\n"
                "- variable scope can be limited\n"
@@ -360,7 +387,7 @@ private:
                "- assignment of a variable to itself\n"
                "- Comparison of values leading always to true or false\n"
                "- Clarify calculation with parentheses\n"
-               "- suspicious comparison of '\\0' with a char* variable\n"
+               "- suspicious comparison of '\\0' with a char\\* variable\n"
                "- duplicate break statement\n"
                "- unreachable code\n"
                "- testing if unsigned variable is negative/positive\n"
@@ -370,8 +397,12 @@ private:
                "- comma in return statement (the comma can easily be misread as a semicolon).\n"
                "- prefer erfc, expm1 or log1p to avoid loss of precision.\n"
                "- identical code in both branches of if/else or ternary operator.\n"
-               "- redundant pointer operation on pointer like &*some_ptr.\n"
-               "- find unused 'goto' labels.\n";
+               "- redundant pointer operation on pointer like &\\*some_ptr.\n"
+               "- find unused 'goto' labels.\n"
+               "- function declaration and definition argument names different.\n"
+               "- function declaration and definition argument order different.\n"
+               "- shadow variable.\n"
+               "- variable can be declared const.\n";
     }
 };
 /// @}
