@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,12 +22,29 @@
 #define checkuninitvarH
 //---------------------------------------------------------------------------
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
+#include "ctu.h"
+#include "errorlogger.h"
+#include "mathlib.h"
+#include "settings.h"
+#include "utils.h"
+
+#include <set>
+#include <string>
 
 class Scope;
+class Token;
+class Tokenizer;
 class Variable;
 
+
+struct VariableValue {
+    explicit VariableValue(MathLib::bigint val = 0) : value(val), notEqual(false) {}
+    MathLib::bigint value;
+    bool notEqual;
+};
+
 /// @addtogroup Checks
 /// @{
 
@@ -45,44 +62,58 @@ public:
         : Check(myName(), tokenizer, settings, errorLogger) {
     }
 
-    /** @brief Run checks against the simplified token list */
-    void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
+    /** @brief Run checks against the normal token list */
+    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) OVERRIDE {
+        if (settings->clang)
+            return;
+
         CheckUninitVar checkUninitVar(tokenizer, settings, errorLogger);
         checkUninitVar.check();
-        checkUninitVar.deadPointer();
+        checkUninitVar.valueFlowUninit();
     }
 
     /** Check for uninitialized variables */
     void check();
-    void checkScope(const Scope* scope);
+    void checkScope(const Scope* scope, const std::set<std::string> &arrayTypeDefs);
     void checkStruct(const Token *tok, const Variable &structvar);
     enum Alloc { NO_ALLOC, NO_CTOR_CALL, CTOR_CALL, ARRAY };
-    bool checkScopeForVariable(const Token *tok, const Variable& var, bool* const possibleInit, bool* const noreturn, Alloc* const alloc, const std::string &membervar);
+    bool checkScopeForVariable(const Token *tok, const Variable& var, bool* const possibleInit, bool* const noreturn, Alloc* const alloc, const std::string &membervar, std::map<int, VariableValue> variableValue);
     bool checkIfForWhileHead(const Token *startparentheses, const Variable& var, bool suppressErrors, bool isuninit, Alloc alloc, const std::string &membervar);
     bool checkLoopBody(const Token *tok, const Variable& var, const Alloc alloc, const std::string &membervar, const bool suppressErrors);
-    void checkRhs(const Token *tok, const Variable &var, Alloc alloc, unsigned int number_of_if, const std::string &membervar);
-    bool isVariableUsage(const Token *vartok, bool ispointer, Alloc alloc) const;
-    int isFunctionParUsage(const Token *vartok, bool ispointer, Alloc alloc) const;
+    void checkRhs(const Token *tok, const Variable &var, Alloc alloc, nonneg int number_of_if, const std::string &membervar);
+    bool isVariableUsage(const Token *vartok, bool pointer, Alloc alloc, int indirect = 0) const;
+    int isFunctionParUsage(const Token *vartok, bool pointer, Alloc alloc, int indirect = 0) const;
     bool isMemberVariableAssignment(const Token *tok, const std::string &membervar) const;
     bool isMemberVariableUsage(const Token *tok, bool isPointer, Alloc alloc, const std::string &membervar) const;
 
-    /** ValueFlow-based checking for dead pointer usage */
-    void deadPointer();
-    void deadPointerError(const Token *pointer, const Token *alias);
+    /** ValueFlow-based checking for uninitialized variables */
+    void valueFlowUninit();
 
     /* data for multifile checking */
     class MyFileInfo : public Check::FileInfo {
     public:
-        /* functions that must have initialized data */
-        std::set<std::string>  uvarFunctions;
+        /** function arguments that data are unconditionally read */
+        std::list<CTU::FileInfo::UnsafeUsage> unsafeUsage;
 
-        /* functions calls with uninitialized data */
-        std::set<std::string>  functionCalls;
+        /** Convert MyFileInfo data into xml string */
+        std::string toString() const OVERRIDE;
     };
 
+    /** @brief Parse current TU and extract file info */
+    Check::FileInfo *getFileInfo(const Tokenizer *tokenizer, const Settings *settings) const OVERRIDE;
+
+    Check::FileInfo * loadFileInfoFromXml(const tinyxml2::XMLElement *xmlElement) const OVERRIDE;
+
+    /** @brief Analyse all file infos for all TU */
+    bool analyseWholeProgram(const CTU::FileInfo *ctu, const std::list<Check::FileInfo*> &fileInfo, const Settings& settings, ErrorLogger &errorLogger) OVERRIDE;
+
     void uninitstringError(const Token *tok, const std::string &varname, bool strncpy_);
     void uninitdataError(const Token *tok, const std::string &varname);
-    void uninitvarError(const Token *tok, const std::string &varname);
+    void uninitvarError(const Token *tok, const std::string &varname, ErrorPath errorPath);
+    void uninitvarError(const Token *tok, const std::string &varname) {
+        ErrorPath errorPath;
+        uninitvarError(tok, varname, errorPath);
+    }
     void uninitvarError(const Token *tok, const std::string &varname, Alloc alloc) {
         if (alloc == NO_CTOR_CALL || alloc == CTOR_CALL)
             uninitdataError(tok, varname);
@@ -92,26 +123,27 @@ public:
     void uninitStructMemberError(const Token *tok, const std::string &membername);
 
 private:
-    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
-        CheckUninitVar c(0, settings, errorLogger);
+    Check::FileInfo *getFileInfo() const;
+    bool isUnsafeFunction(const Scope *scope, int argnr, const Token **tok) const;
+
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const OVERRIDE {
+        CheckUninitVar c(nullptr, settings, errorLogger);
 
         // error
-        c.uninitstringError(0, "varname", true);
-        c.uninitdataError(0, "varname");
-        c.uninitvarError(0, "varname");
-        c.uninitStructMemberError(0, "a.b");
-        c.deadPointerError(0,0);
+        c.uninitstringError(nullptr, "varname", true);
+        c.uninitdataError(nullptr, "varname");
+        c.uninitvarError(nullptr, "varname");
+        c.uninitStructMemberError(nullptr, "a.b");
     }
 
     static std::string myName() {
         return "Uninitialized variables";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const OVERRIDE {
         return "Uninitialized variables\n"
                "- using uninitialized local variables\n"
-               "- using allocated data before it has been initialized\n"
-               "- using dead pointer\n";
+               "- using allocated data before it has been initialized\n";
     }
 };
 /// @}
