@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2018 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ public:
     /**
      * @brief Destructor.
      */
-    virtual ~CppCheck();
+    ~CppCheck() OVERRIDE;
 
     /**
      * @brief This starts the actual checking. Note that you must call
@@ -113,7 +113,7 @@ public:
      * @brief Terminate checking. The checking will be terminated as soon as possible.
      */
     void terminate() {
-        _settings.terminate();
+        Settings::terminate();
     }
 
     /**
@@ -126,7 +126,7 @@ public:
     void purgedConfigurationMessage(const std::string &file, const std::string& configuration);
 
     void dontSimplify() {
-        _simplify = false;
+        mSimplify = false;
     }
 
     /** Analyse whole program, run this after all TUs has been scanned.
@@ -136,6 +136,9 @@ public:
      */
     bool analyseWholeProgram();
 
+    /** Analyze all files using clang-tidy */
+    void analyseClangTidy(const ImportProject::FileSettings &fileSettings);
+
     /** analyse whole program use .analyzeinfo files */
     void analyseWholeProgram(const std::string &buildDir, const std::map<std::string, std::size_t> &files);
 
@@ -145,6 +148,9 @@ public:
 
 private:
 
+    /** Are there "simple" rules */
+    bool hasRule(const std::string &tokenlist) const;
+
     /** @brief There has been an internal error => Report information message */
     void internalError(const std::string &filename, const std::string &msg);
 
@@ -169,12 +175,6 @@ private:
      */
     void checkNormalTokens(const Tokenizer &tokenizer);
 
-    /**
-     * @brief Check simplified tokens
-     * @param tokenizer tokenizer instance
-     */
-    void checkSimplifiedTokens(const Tokenizer &tokenizer);
-
     /**
      * @brief Execute rules, if any
      * @param tokenlist token list to use (normal / simple)
@@ -189,44 +189,48 @@ private:
      * "[filepath:line number] Message", e.g.
      * "[main.cpp:4] Uninitialized member variable"
      */
-    virtual void reportErr(const ErrorLogger::ErrorMessage &msg) override;
+    void reportErr(const ErrorLogger::ErrorMessage &msg) OVERRIDE;
 
     /**
      * @brief Information about progress is directed here.
      *
      * @param outmsg Message to show, e.g. "Checking main.cpp..."
      */
-    virtual void reportOut(const std::string &outmsg) override;
+    void reportOut(const std::string &outmsg) OVERRIDE;
 
-    std::list<std::string> _errorList;
-    Settings _settings;
+    void bughuntingReport(const std::string &str) OVERRIDE;
 
-    void reportProgress(const std::string &filename, const char stage[], const std::size_t value) override;
+    std::list<std::string> mErrorList;
+    Settings mSettings;
+
+    void reportProgress(const std::string &filename, const char stage[], const std::size_t value) OVERRIDE;
 
     /**
      * Output information messages.
      */
-    virtual void reportInfo(const ErrorLogger::ErrorMessage &msg) override;
+    void reportInfo(const ErrorLogger::ErrorMessage &msg) OVERRIDE;
 
-    ErrorLogger &_errorLogger;
+    ErrorLogger &mErrorLogger;
 
     /** @brief Current preprocessor configuration */
-    std::string cfg;
+    std::string mCurrentConfig;
 
-    unsigned int exitcode;
+    unsigned int mExitCode;
 
-    bool _useGlobalSuppressions;
+    bool mSuppressInternalErrorFound;
+
+    bool mUseGlobalSuppressions;
 
     /** Are there too many configs? */
-    bool tooManyConfigs;
+    bool mTooManyConfigs;
 
     /** Simplify code? true by default */
-    bool _simplify;
+    bool mSimplify;
 
     /** File info used for whole program analysis */
-    std::list<Check::FileInfo*> fileInfo;
+    std::list<Check::FileInfo*> mFileInfo;
 
-    AnalyzerInformation analyzerInformation;
+    AnalyzerInformation mAnalyzerInformation;
 };
 
 /// @}
