@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2018 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,10 +36,16 @@
  * See https://cwe.mitre.org/ for further reference.
  * */
 struct CWE {
-    explicit CWE(unsigned short ID) : id(ID) {}
+    explicit CWE(unsigned short cweId) : id(cweId) {}
     unsigned short id;
 };
 
+// CWE list: https://cwe.mitre.org/data/published/cwe_v3.4.1.pdf
+static const struct CWE CWE_USE_OF_UNINITIALIZED_VARIABLE(457U);
+static const struct CWE CWE_NULL_POINTER_DEREFERENCE(476U);
+static const struct CWE CWE_USE_OF_POTENTIALLY_DANGEROUS_FUNCTION(676U);
+static const struct CWE CWE_INCORRECT_CALCULATION(682U);
+static const struct CWE CWE_EXPIRED_POINTER_DEREFERENCE(825U);
 
 
 class Token;
@@ -54,10 +60,11 @@ namespace tinyxml2 {
 
 /** @brief Simple container to be thrown when internal error is detected. */
 struct InternalError {
-    enum Type {AST, SYNTAX, INTERNAL};
+    enum Type {AST, SYNTAX, UNKNOWN_MACRO, INTERNAL, LIMIT, INSTANTIATION};
     InternalError(const Token *tok, const std::string &errorMsg, Type type = INTERNAL);
     const Token *token;
     std::string errorMessage;
+    Type type;
     std::string id;
 };
 
@@ -137,7 +144,7 @@ public:
             return "information";
         case debug:
             return "debug";
-        };
+        }
         throw InternalError(nullptr, "Unknown severity");
     }
     static SeverityType fromString(const std::string &severity) {
@@ -189,15 +196,15 @@ public:
         class CPPCHECKLIB FileLocation {
         public:
             FileLocation()
-                : fileIndex(0), line(0), col(0) {
+                : fileIndex(0), line(0), column(0) {
             }
 
-            FileLocation(const std::string &file, unsigned int aline)
-                : fileIndex(0), line(aline), col(0), _file(file) {
+            FileLocation(const std::string &file, int line, int column)
+                : fileIndex(0), line(line), column(column), mOrigFileName(file), mFileName(file) {
             }
 
-            FileLocation(const std::string &file, const std::string &info, unsigned int aline)
-                : fileIndex(0), line(aline), col(0), _file(file), _info(info) {
+            FileLocation(const std::string &file, const std::string &info, int line, int column)
+                : fileIndex(0), line(line), column(column), mOrigFileName(file), mFileName(file), mInfo(info) {
             }
 
             FileLocation(const Token* tok, const TokenList* tokenList);
@@ -210,6 +217,13 @@ public:
              */
             std::string getfile(bool convert = true) const;
 
+            /**
+             * Filename with the whole path (no --rp)
+             * @param convert If true convert path to native separators.
+             * @return filename.
+             */
+            std::string getOrigFile(bool convert = true) const;
+
             /**
              * Set the filename.
              * @param file Filename to set.
@@ -222,26 +236,54 @@ public:
             std::string stringify() const;
 
             unsigned int fileIndex;
-            unsigned int line;
-            unsigned int col;
+            int line; // negative value means "no line"
+            unsigned int column;
 
             std::string getinfo() const {
-                return _info;
+                return mInfo;
             }
             void setinfo(const std::string &i) {
-                _info = i;
+                mInfo = i;
             }
 
         private:
-            std::string _file;
-            std::string _info;
+            std::string mOrigFileName;
+            std::string mFileName;
+            std::string mInfo;
         };
 
-        ErrorMessage(const std::list<FileLocation> &callStack, const std::string& file1, Severity::SeverityType severity, const std::string &msg, const std::string &id, bool inconclusive);
-        ErrorMessage(const std::list<FileLocation> &callStack, const std::string& file1, Severity::SeverityType severity, const std::string &msg, const std::string &id, const CWE &cwe, bool inconclusive);
-        ErrorMessage(const std::list<const Token*>& callstack, const TokenList* list, Severity::SeverityType severity, const std::string& id, const std::string& msg, bool inconclusive);
-        ErrorMessage(const std::list<const Token*>& callstack, const TokenList* list, Severity::SeverityType severity, const std::string& id, const std::string& msg, const CWE &cwe, bool inconclusive);
-        ErrorMessage(const ErrorPath &errorPath, const TokenList *tokenList, Severity::SeverityType severity, const char id[], const std::string &msg, const CWE &cwe, bool inconclusive);
+        ErrorMessage(const std::list<FileLocation> &callStack,
+                     const std::string& file1,
+                     Severity::SeverityType severity,
+                     const std::string &msg,
+                     const std::string &id, bool inconclusive);
+        ErrorMessage(const std::list<FileLocation> &callStack,
+                     const std::string& file1,
+                     Severity::SeverityType severity,
+                     const std::string &msg,
+                     const std::string &id,
+                     const CWE &cwe,
+                     bool inconclusive);
+        ErrorMessage(const std::list<const Token*>& callstack,
+                     const TokenList* list,
+                     Severity::SeverityType severity,
+                     const std::string& id,
+                     const std::string& msg,
+                     bool inconclusive);
+        ErrorMessage(const std::list<const Token*>& callstack,
+                     const TokenList* list,
+                     Severity::SeverityType severity,
+                     const std::string& id,
+                     const std::string& msg,
+                     const CWE &cwe,
+                     bool inconclusive);
+        ErrorMessage(const ErrorPath &errorPath,
+                     const TokenList *tokenList,
+                     Severity::SeverityType severity,
+                     const char id[],
+                     const std::string &msg,
+                     const CWE &cwe,
+                     bool inconclusive);
         ErrorMessage();
         explicit ErrorMessage(const tinyxml2::XMLElement * const errmsg);
 
@@ -262,37 +304,43 @@ public:
          * or template to be used. E.g. "{file}:{line},{info}"
         * @return formatted string
          */
-        std::string toString(bool verbose, const std::string &templateFormat = emptyString, const std::string &templateLocation = emptyString) const;
+        std::string toString(bool verbose,
+                             const std::string &templateFormat = emptyString,
+                             const std::string &templateLocation = emptyString) const;
 
         std::string serialize() const;
         bool deserialize(const std::string &data);
 
-        std::list<FileLocation> _callStack;
-        std::string _id;
+        std::list<FileLocation> callStack;
+        std::string id;
 
-        /** source file (not header) */
+        /** For GUI rechecking; source file (not header) */
         std::string file0;
+        /** For GUI bug hunting; function name */
+        std::string function;
+        /** For GUI bug hunting; incomplete analysis */
+        bool incomplete;
 
-        Severity::SeverityType _severity;
-        CWE _cwe;
-        bool _inconclusive;
+        Severity::SeverityType severity;
+        CWE cwe;
+        bool inconclusive;
 
         /** set short and verbose messages */
         void setmsg(const std::string &msg);
 
         /** Short message (single line short message) */
         const std::string &shortMessage() const {
-            return _shortMessage;
+            return mShortMessage;
         }
 
         /** Verbose message (may be the same as the short message) */
         const std::string &verboseMessage() const {
-            return _verboseMessage;
+            return mVerboseMessage;
         }
 
         /** Symbol names */
         const std::string &symbolNames() const {
-            return _symbolNames;
+            return mSymbolNames;
         }
 
         Suppressions::ErrorMessage toSuppressionsErrorMessage() const;
@@ -310,13 +358,13 @@ public:
         static std::string fixInvalidChars(const std::string& raw);
 
         /** Short message */
-        std::string _shortMessage;
+        std::string mShortMessage;
 
         /** Verbose message */
-        std::string _verboseMessage;
+        std::string mVerboseMessage;
 
         /** symbol names */
-        std::string _symbolNames;
+        std::string mSymbolNames;
     };
 
     ErrorLogger() { }
@@ -363,11 +411,14 @@ public:
         reportErr(msg);
     }
 
+    virtual void bughuntingReport(const std::string &str) = 0;
+
     /**
-     * Report list of unmatched suppressions
+     * Report unmatched suppressions
      * @param unmatched list of unmatched suppressions (from Settings::Suppressions::getUnmatched(Local|Global)Suppressions)
+     * @return true is returned if errors are reported
      */
-    void reportUnmatchedSuppressions(const std::list<Suppressions::Suppression> &unmatched);
+    bool reportUnmatchedSuppressions(const std::list<Suppressions::Suppression> &unmatched);
 
     static std::string callStackToString(const std::list<ErrorLogger::ErrorMessage::FileLocation> &callStack);
 
@@ -387,6 +438,9 @@ public:
     }
 };
 
+/** Replace substring. Example replaceStr("1,NR,3", "NR", "2") => "1,2,3" */
+std::string replaceStr(std::string s, const std::string &from, const std::string &to);
+
 /// @}
 //---------------------------------------------------------------------------
 #endif // errorloggerH
