@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,18 @@
 #define tokenlistH
 //---------------------------------------------------------------------------
 
+#include "config.h"
+#include "token.h"
+
 #include <string>
 #include <vector>
-#include "config.h"
 
-class Token;
 class Settings;
 
+namespace simplecpp {
+    class TokenList;
+}
+
 /// @addtogroup Core
 /// @{
 
@@ -37,7 +42,11 @@ public:
     ~TokenList();
 
     void setSettings(const Settings *settings) {
-        _settings = settings;
+        mSettings = settings;
+    }
+
+    const Settings *getSettings() const {
+        return mSettings;
     }
 
     /** @return the source file path. e.g. "file.cpp" */
@@ -45,12 +54,12 @@ public:
 
     /** Is the code C. Used for bailouts */
     bool isC() const {
-        return _isC;
+        return mIsC;
     }
 
     /** Is the code CPP. Used for bailouts */
     bool isCPP() const {
-        return _isCPP;
+        return mIsCpp;
     }
 
     /**
@@ -59,10 +68,24 @@ public:
      */
     static void deleteTokens(Token *tok);
 
-    void addtoken(std::string str, const unsigned int lineno, const unsigned int fileno, bool split = false);
-    void addtoken(const Token *tok, const unsigned int lineno, const unsigned int fileno);
+    void addtoken(std::string str, const nonneg int lineno, const nonneg int fileno, bool split = false);
+    void addtoken(std::string str, const Token *locationTok);
 
-    static void insertTokens(Token *dest, const Token *src, unsigned int n);
+    void addtoken(const Token *tok, const nonneg int lineno, const nonneg int fileno);
+    void addtoken(const Token *tok, const Token *locationTok);
+    void addtoken(const Token *tok);
+
+    static void insertTokens(Token *dest, const Token *src, nonneg int n);
+
+    /**
+     * Copy tokens.
+     * @param dest destination token where copied tokens will be inserted after
+     * @param first first token to copy
+     * @param last last token to copy
+     * @param one_line true=>copy all tokens to the same line as dest. false=>copy all tokens to dest while keeping the 'line breaks'
+     * @return new location of last token copied
+     */
+    static Token *copyTokens(Token *dest, const Token *first, const Token *last, bool one_line = true);
 
     /**
      * Create tokens from code.
@@ -75,26 +98,28 @@ public:
      */
     bool createTokens(std::istream &code, const std::string& file0 = emptyString);
 
+    void createTokens(const simplecpp::TokenList *tokenList);
+
     /** Deallocate list */
     void deallocateTokens();
 
     /** append file name if seen the first time; return its index in any case */
-    unsigned int appendFileIfNew(const std::string &file);
+    int appendFileIfNew(const std::string &fileName);
 
     /** get first token of list */
     const Token *front() const {
-        return _front;
+        return mTokensFrontBack.front;
     }
     Token *front() {
-        return _front;
+        return mTokensFrontBack.front;
     }
 
     /** get last token of list */
     const Token *back() const {
-        return _back;
+        return mTokensFrontBack.back;
     }
     Token *back() {
-        return _back;
+        return mTokensFrontBack.back;
     }
 
     /**
@@ -103,9 +128,11 @@ public:
      * @return vector with filenames
      */
     const std::vector<std::string>& getFiles() const {
-        return _files;
+        return mFiles;
     }
 
+    std::string getOrigFile(const Token *tok) const;
+
     /**
      * get filename for given token
      * @param tok The given token
@@ -135,7 +162,7 @@ public:
      * Check abstract syntax tree.
      * Throws InternalError on failure
      */
-    void validateAst();
+    void validateAst() const;
 
     /**
      * Verify that the given token is an element of the tokenlist.
@@ -145,6 +172,23 @@ public:
      */
     bool validateToken(const Token* tok) const;
 
+    /**
+     * Convert platform dependent types to standard types.
+     * 32 bits: size_t -> unsigned long
+     * 64 bits: size_t -> unsigned long long
+     */
+    void simplifyPlatformTypes();
+
+    /**
+     * Collapse compound standard types into a single token.
+     * unsigned long long int => long _isUnsigned=true,_isLong=true
+     */
+    void simplifyStdType();
+
+    void clangSetOrigFiles();
+
+    bool isKeyword(const std::string &str) const;
+
 private:
 
     /** Disable copy constructor, no implementation */
@@ -154,16 +198,22 @@ private:
     TokenList &operator=(const TokenList &);
 
     /** Token list */
-    Token *_front, *_back;
+    TokensFrontBack mTokensFrontBack;
 
     /** filenames for the tokenized source code (source + included) */
-    std::vector<std::string> _files;
+    std::vector<std::string> mFiles;
+
+    /** Original filenames for the tokenized source code (source + included) */
+    std::vector<std::string> mOrigFiles;
 
     /** settings */
-    const Settings* _settings;
+    const Settings* mSettings;
+
+    std::set<std::string> mKeywords;
 
     /** File is known to be C/C++ code */
-    bool _isC, _isCPP;
+    bool mIsC;
+    bool mIsCpp;
 };
 
 /// @}
