@@ -1,6 +1,6 @@
 ﻿/*
 * Cppcheck - A tool for static C/C++ code analysis
-* Copyright (C) 2007-2018 Cppcheck team.
+* Copyright (C) 2007-2020 Cppcheck team.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,13 +16,14 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include "cppcheckexecutor.h"
 #include "errorlogger.h"
-#include "cppcheck.h"
 #include "filelister.h"
 #include "path.h"
 #include "pathmatch.h"
 #include "redirect.h"
+#include "settings.h"
 #include "testsuite.h"
 
 #include <algorithm>
@@ -44,7 +45,7 @@ public:
 
 private:
 
-    void run() override {
+    void run() OVERRIDE {
         TEST_CASE(runSamples);
         TEST_CASE(runConsoleCodePageTranslationOnWindows);
     }
@@ -61,10 +62,11 @@ private:
         FileLister::recursiveAddFiles(files, "samples", matcher);
 #endif
         for (std::map<std::string, std::size_t>::const_iterator i = files.begin(); i != files.end(); ++i) {
+            if (i->first.find("memleak") != std::string::npos)
+                continue;
             CLEAR_REDIRECT_ERROUT;
-            char* path = new char[i->first.size() + 1];
-            strcpy(path, i->first.c_str());
-            const char* argv[] = {
+            const char* const path = i->first.c_str();
+            const char * const argv[] = {
 #ifdef _WIN32
                 ".\\..\\testrunner",
 #else
@@ -90,7 +92,6 @@ private:
                     actual.erase(actual.find("..\\"), 3); // Remove '..\'
                 ASSERT_EQUALS_MSG(expected, actual, i->first);
             }
-            delete[] path;
         }
     }
 
