@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2020 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,13 +16,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "settings.h"
 #include "testsuite.h"
 #include "testutils.h"
 #include "token.h"
-#include "settings.h"
+#include "tokenize.h"
+#include "tokenlist.h"
 
-#include <vector>
 #include <string>
+#include <vector>
+
+struct InternalError;
 
 
 class TestToken : public TestFixture {
@@ -38,8 +42,13 @@ private:
     std::vector<std::string> extendedOps;
     std::vector<std::string> assignmentOps;
 
-    void run() {
-        initOps();
+    void run() OVERRIDE {
+        arithmeticalOps = { "+", "-", "*", "/", "%", "<<", ">>" };
+        logicalOps = { "&&", "||", "!" };
+        comparisonOps = { "==", "!=", "<", "<=", ">", ">=" };
+        bitOps = { "&", "|", "^", "~" };
+        extendedOps = { ",", "[", "]", "(", ")", "?", ":" };
+        assignmentOps = { "=", "+=", "-=", "*=", "/=", "%=", "&=", "^=", "|=", "<<=", ">>=" };
 
         TEST_CASE(nextprevious);
         TEST_CASE(multiCompare);
@@ -47,11 +56,16 @@ private:
         TEST_CASE(multiCompare3);                   // false positive for %or% on code using "|="
         TEST_CASE(multiCompare4);
         TEST_CASE(multiCompare5);
+        TEST_CASE(charTypes);
+        TEST_CASE(stringTypes);
         TEST_CASE(getStrLength);
         TEST_CASE(getStrSize);
+        TEST_CASE(getCharAt);
         TEST_CASE(strValue);
+        TEST_CASE(concatStr);
 
         TEST_CASE(deleteLast);
+        TEST_CASE(deleteFirst);
         TEST_CASE(nextArgument);
         TEST_CASE(eraseTokens);
 
@@ -78,23 +92,28 @@ private:
         TEST_CASE(literals);
         TEST_CASE(operators);
 
-        TEST_CASE(updateProperties)
-        TEST_CASE(updatePropertiesConcatStr)
-        TEST_CASE(isNameGuarantees1)
-        TEST_CASE(isNameGuarantees2)
-        TEST_CASE(isNameGuarantees3)
-        TEST_CASE(isNameGuarantees4)
-        TEST_CASE(isNameGuarantees5)
+        TEST_CASE(updateProperties);
+        TEST_CASE(isNameGuarantees1);
+        TEST_CASE(isNameGuarantees2);
+        TEST_CASE(isNameGuarantees3);
+        TEST_CASE(isNameGuarantees4);
+        TEST_CASE(isNameGuarantees5);
+        TEST_CASE(isNameGuarantees6);
 
         TEST_CASE(canFindMatchingBracketsNeedsOpen);
         TEST_CASE(canFindMatchingBracketsInnerPair);
         TEST_CASE(canFindMatchingBracketsOuterPair);
         TEST_CASE(canFindMatchingBracketsWithTooManyClosing);
         TEST_CASE(canFindMatchingBracketsWithTooManyOpening);
+        TEST_CASE(findClosingBracket);
+
+        TEST_CASE(expressionString);
+
+        TEST_CASE(hasKnownIntValue);
     }
 
     void nextprevious() const {
-        Token *token = new Token(0);
+        Token *token = new Token();
         token->str("1");
         token->insertToken("2");
         token->next()->insertToken("3");
@@ -102,14 +121,12 @@ private:
         ASSERT_EQUALS(token->str(), "1");
         ASSERT_EQUALS(token->next()->str(), "2");
         ASSERT_EQUALS(token->tokAt(2)->str(), "3");
-        if (last->next())
-            ASSERT_EQUALS("Null was expected", "");
+        ASSERT_EQUALS_MSG(true, last->next() == nullptr, "Null was expected");
 
         ASSERT_EQUALS(last->str(), "3");
         ASSERT_EQUALS(last->previous()->str(), "2");
         ASSERT_EQUALS(last->tokAt(-2)->str(), "1");
-        if (token->previous())
-            ASSERT_EQUALS("Null was expected", "");
+        ASSERT_EQUALS_MSG(true, token->previous() == nullptr, "Null was expected");
 
         TokenList::deleteTokens(token);
     }
@@ -117,58 +134,58 @@ private:
     bool Match(const std::string &code, const std::string &pattern, unsigned int varid=0) {
         static const Settings settings;
         Tokenizer tokenizer(&settings, this);
-        std::istringstream istr(code);
+        std::istringstream istr(";" + code + ";");
         try {
             tokenizer.tokenize(istr, "test.cpp");
         } catch (...) {}
-        return Token::Match(tokenizer.tokens(), pattern.c_str(), varid);
+        return Token::Match(tokenizer.tokens()->next(), pattern.c_str(), varid);
     }
 
     void multiCompare() const {
         // Test for found
-        Token one(0);
+        Token one;
         one.str("one");
         ASSERT_EQUALS(1, Token::multiCompare(&one, "one|two", 0));
 
-        Token two(0);
+        Token two;
         two.str("two");
         ASSERT_EQUALS(1, Token::multiCompare(&two, "one|two", 0));
         ASSERT_EQUALS(1, Token::multiCompare(&two, "verybig|two|", 0));
 
         // Test for empty string found
-        Token notfound(0);
+        Token notfound;
         notfound.str("notfound");
         ASSERT_EQUALS(0, Token::multiCompare(&notfound, "one|two|", 0));
 
         // Test for not found
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&notfound, "one|two", 0)));
 
-        Token s(0);
+        Token s;
         s.str("s");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&s, "verybig|two", 0)));
 
-        Token ne(0);
+        Token ne;
         ne.str("ne");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&ne, "one|two", 0)));
 
-        Token a(0);
+        Token a;
         a.str("a");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&a, "abc|def", 0)));
 
-        Token abcd(0);
+        Token abcd;
         abcd.str("abcd");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&abcd, "abc|def", 0)));
 
-        Token def(0);
+        Token def;
         def.str("default");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&def, "abc|def", 0)));
 
         // %op%
-        Token plus(0);
+        Token plus;
         plus.str("+");
         ASSERT_EQUALS(1, Token::multiCompare(&plus, "one|%op%", 0));
         ASSERT_EQUALS(1, Token::multiCompare(&plus, "%op%|two", 0));
-        Token x(0);
+        Token x;
         x.str("x");
         ASSERT_EQUALS(-1, Token::multiCompare(&x, "one|%op%", 0));
         ASSERT_EQUALS(-1, Token::multiCompare(&x, "%op%|two", 0));
@@ -244,42 +261,176 @@ private:
     }
 
     void multiCompare5() const {
-        Token tok(0);
+        Token tok;
         tok.str("||");
         ASSERT_EQUALS(true, Token::multiCompare(&tok, "+|%or%|%oror%", 0) >= 0);
     }
 
+    void charTypes() const {
+        Token tok;
+
+        tok.str("'a'");
+        ASSERT_EQUALS(true, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("u8'a'");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(true, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("u'a'");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(true, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("U'a'");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(true, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("L'a'");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(true, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("'aaa'");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(true, tok.isCMultiChar());
+    }
+
+    void stringTypes() const {
+        Token tok;
+
+        tok.str("\"a\"");
+        ASSERT_EQUALS(true, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("u8\"a\"");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(true, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("u\"a\"");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(true, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("U\"a\"");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(true, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("L\"a\"");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(true, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+    }
+
     void getStrLength() const {
-        Token tok(0);
+        Token tok;
 
         tok.str("\"\"");
-        ASSERT_EQUALS(0, (int)Token::getStrLength(&tok));
+        ASSERT_EQUALS(0, Token::getStrLength(&tok));
 
         tok.str("\"test\"");
-        ASSERT_EQUALS(4, (int)Token::getStrLength(&tok));
+        ASSERT_EQUALS(4, Token::getStrLength(&tok));
 
         tok.str("\"test \\\\test\"");
-        ASSERT_EQUALS(10, (int)Token::getStrLength(&tok));
+        ASSERT_EQUALS(10, Token::getStrLength(&tok));
 
         tok.str("\"a\\0\"");
-        ASSERT_EQUALS(1, (int)Token::getStrLength(&tok));
+        ASSERT_EQUALS(1, Token::getStrLength(&tok));
+
+        tok.str("L\"\"");
+        ASSERT_EQUALS(0, Token::getStrLength(&tok));
+
+        tok.str("u8\"test\"");
+        ASSERT_EQUALS(4, Token::getStrLength(&tok));
+
+        tok.str("U\"test \\\\test\"");
+        ASSERT_EQUALS(10, Token::getStrLength(&tok));
+
+        tok.str("u\"a\\0\"");
+        ASSERT_EQUALS(1, Token::getStrLength(&tok));
     }
 
     void getStrSize() const {
-        Token tok(0);
+        Token tok;
+        Settings settings;
+
+        tok.str("\"\"");
+        ASSERT_EQUALS(sizeof(""), Token::getStrSize(&tok, &settings));
 
         tok.str("\"abc\"");
-        ASSERT_EQUALS(sizeof("abc"), Token::getStrSize(&tok));
+        ASSERT_EQUALS(sizeof("abc"), Token::getStrSize(&tok, &settings));
 
         tok.str("\"\\0abc\"");
-        ASSERT_EQUALS(sizeof("\0abc"), Token::getStrSize(&tok));
+        ASSERT_EQUALS(sizeof("\0abc"), Token::getStrSize(&tok, &settings));
 
         tok.str("\"\\\\\"");
-        ASSERT_EQUALS(sizeof("\\"), Token::getStrSize(&tok));
+        ASSERT_EQUALS(sizeof("\\"), Token::getStrSize(&tok, &settings));
+    }
+
+    void getCharAt() const {
+        Token tok;
+
+        tok.str("\"asdf\"");
+        ASSERT_EQUALS("a", Token::getCharAt(&tok, 0));
+        ASSERT_EQUALS("s", Token::getCharAt(&tok, 1));
+
+        tok.str("\"a\\ts\"");
+        ASSERT_EQUALS("\\t", Token::getCharAt(&tok, 1));
+
+        tok.str("\"\"");
+        ASSERT_EQUALS("\\0", Token::getCharAt(&tok, 0));
+
+        tok.str("L\"a\\ts\"");
+        ASSERT_EQUALS("a", Token::getCharAt(&tok, 0));
+        ASSERT_EQUALS("\\t", Token::getCharAt(&tok, 1));
+
+        tok.str("u\"a\\ts\"");
+        ASSERT_EQUALS("\\t", Token::getCharAt(&tok, 1));
+        ASSERT_EQUALS("s", Token::getCharAt(&tok, 2));
     }
 
     void strValue() const {
-        Token tok(0);
+        Token tok;
 
         tok.str("\"\"");
         ASSERT_EQUALS("", tok.strValue());
@@ -302,16 +453,72 @@ private:
         tok.str("\"a\\0\"");
         ASSERT_EQUALS("a", tok.strValue());
 
+        tok.str("L\"a\\t\"");
+        ASSERT_EQUALS("a\t", tok.strValue());
+
+        tok.str("U\"a\\0\"");
+        ASSERT_EQUALS("a", tok.strValue());
     }
 
+    void concatStr() const {
+        Token tok;
+
+        tok.str("\"\"");
+        tok.concatStr("\"\"");
+        ASSERT_EQUALS("", tok.strValue());
+        ASSERT(tok.isCChar());
+
+        tok.str("\"ab\"");
+        tok.concatStr("\"cd\"");
+        ASSERT_EQUALS("abcd", tok.strValue());
+        ASSERT(tok.isCChar());
+
+        tok.str("L\"ab\"");
+        tok.concatStr("L\"cd\"");
+        ASSERT_EQUALS("abcd", tok.strValue());
+        ASSERT(tok.isLong());
+
+        tok.str("L\"ab\"");
+        tok.concatStr("\"cd\"");
+        ASSERT_EQUALS("abcd", tok.strValue());
+        ASSERT(tok.isLong());
+
+        tok.str("\"ab\"");
+        tok.concatStr("L\"cd\"");
+        ASSERT_EQUALS("abcd", tok.strValue());
+        ASSERT(tok.isLong());
+
+        tok.str("\"ab\"");
+        tok.concatStr("L\"\"");
+        ASSERT_EQUALS("ab", tok.strValue());
+        ASSERT(tok.isLong());
+
+        tok.str("\"ab\"");
+        tok.concatStr("u8\"cd\"");
+        ASSERT_EQUALS("abcd", tok.strValue());
+        ASSERT(tok.isUtf8());
+    }
 
     void deleteLast() const {
-        Token *tokensBack = 0;
-        Token tok(&tokensBack);
+        TokensFrontBack listEnds{ nullptr };
+        Token ** const tokensBack = &(listEnds.back);
+        Token tok(&listEnds);
         tok.insertToken("aba");
-        ASSERT_EQUALS(true, tokensBack == tok.next());
+        ASSERT_EQUALS(true, *tokensBack == tok.next());
         tok.deleteNext();
-        ASSERT_EQUALS(true, tokensBack == &tok);
+        ASSERT_EQUALS(true, *tokensBack == &tok);
+    }
+
+    void deleteFirst() const {
+        TokensFrontBack listEnds{ nullptr };
+        Token ** const tokensFront = &(listEnds.front);
+        Token tok(&listEnds);
+
+        tok.insertToken("aba");
+
+        ASSERT_EQUALS(true, *tokensFront == tok.previous());
+        tok.deletePrevious();
+        ASSERT_EQUALS(true, *tokensFront == &tok);
     }
 
     void nextArgument() const {
@@ -320,7 +527,7 @@ private:
         ASSERT_EQUALS(true, Token::simpleMatch(example1.tokens()->tokAt(4)->nextArgument(), "3 , 4"));
 
         givenACodeSampleToTokenize example2("foo();");
-        ASSERT_EQUALS(true, example2.tokens()->tokAt(2)->nextArgument() == 0);
+        ASSERT_EQUALS(true, example2.tokens()->tokAt(2)->nextArgument() == nullptr);
 
         givenACodeSampleToTokenize example3("foo(bar(a, b), 2, 3);");
         ASSERT_EQUALS(true, Token::simpleMatch(example3.tokens()->tokAt(2)->nextArgument(), "2 , 3"));
@@ -332,7 +539,7 @@ private:
     void eraseTokens() const {
         givenACodeSampleToTokenize code("begin ; { this code will be removed } end", true);
         Token::eraseTokens(code.tokens()->next(), code.tokens()->tokAt(9));
-        ASSERT_EQUALS("begin ; end", code.tokens()->stringifyList(0, false));
+        ASSERT_EQUALS("begin ; end", code.tokens()->stringifyList(nullptr, false));
     }
 
 
@@ -349,7 +556,7 @@ private:
         ASSERT_EQUALS(true, Token::Match(singleChar.tokens(), "[a|bc]"));
         ASSERT_EQUALS(false, Token::Match(singleChar.tokens(), "[d|ef]"));
 
-        Token multiChar(0);
+        Token multiChar;
         multiChar.str("[ab");
         ASSERT_EQUALS(false, Token::Match(&multiChar, "[ab|def]"));
     }
@@ -432,7 +639,7 @@ private:
         givenACodeSampleToTokenize var("int a ; int b ;");
 
         // Varid == 0 should throw exception
-        ASSERT_THROW(Token::Match(var.tokens(), "%type% %varid% ; %type% %name%", 0),InternalError);
+        ASSERT_THROW((void)Token::Match(var.tokens(), "%type% %varid% ; %type% %name%", 0),InternalError);
 
         ASSERT_EQUALS(true, Token::Match(var.tokens(), "%type% %varid% ; %type% %name%", 1));
         ASSERT_EQUALS(true, Token::Match(var.tokens(), "%type% %name% ; %type% %varid%", 2));
@@ -511,77 +718,33 @@ private:
     }
 
     void matchOr() const {
-        givenACodeSampleToTokenize bitwiseOr("|", true);
-        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "%or%"));
-        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "%op%"));
-        ASSERT_EQUALS(false, Token::Match(bitwiseOr.tokens(), "%oror%"));
+        givenACodeSampleToTokenize bitwiseOr(";|;", true);
+        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "; %or%"));
+        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "; %op%"));
+        ASSERT_EQUALS(false, Token::Match(bitwiseOr.tokens(), "; %oror%"));
 
-        givenACodeSampleToTokenize bitwiseOrAssignment("|=");
-        ASSERT_EQUALS(false,  Token::Match(bitwiseOrAssignment.tokens(), "%or%"));
-        ASSERT_EQUALS(true,  Token::Match(bitwiseOrAssignment.tokens(), "%op%"));
-        ASSERT_EQUALS(false, Token::Match(bitwiseOrAssignment.tokens(), "%oror%"));
+        givenACodeSampleToTokenize bitwiseOrAssignment(";|=;");
+        ASSERT_EQUALS(false,  Token::Match(bitwiseOrAssignment.tokens(), "; %or%"));
+        ASSERT_EQUALS(true,  Token::Match(bitwiseOrAssignment.tokens(), "; %op%"));
+        ASSERT_EQUALS(false, Token::Match(bitwiseOrAssignment.tokens(), "; %oror%"));
 
-        givenACodeSampleToTokenize logicalOr("||", true);
-        ASSERT_EQUALS(false, Token::Match(logicalOr.tokens(), "%or%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "%op%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "%oror%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "&&|%oror%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "%oror%|&&"));
+        givenACodeSampleToTokenize logicalOr(";||;", true);
+        ASSERT_EQUALS(false, Token::Match(logicalOr.tokens(), "; %or%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; %op%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; %oror%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; &&|%oror%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; %oror%|&&"));
 
-        givenACodeSampleToTokenize logicalAnd("&&", true);
-        ASSERT_EQUALS(true, Token::simpleMatch(logicalAnd.tokens(), "&&"));
-        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "&&|%oror%"));
-        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "%oror%|&&"));
+        givenACodeSampleToTokenize logicalAnd(";&&;", true);
+        ASSERT_EQUALS(true, Token::simpleMatch(logicalAnd.tokens(), "; &&"));
+        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "; &&|%oror%"));
+        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "; %oror%|&&"));
     }
 
     static void append_vector(std::vector<std::string> &dest, const std::vector<std::string> &src) {
         dest.insert(dest.end(), src.begin(), src.end());
     }
 
-    void initOps() {
-        arithmeticalOps.push_back("+");
-        arithmeticalOps.push_back("-");
-        arithmeticalOps.push_back("*");
-        arithmeticalOps.push_back("/");
-        arithmeticalOps.push_back("%");
-        arithmeticalOps.push_back("<<");
-        arithmeticalOps.push_back(">>");
-
-        logicalOps.push_back("&&");
-        logicalOps.push_back("||");
-        logicalOps.push_back("!");
-        comparisonOps.push_back("==");
-        comparisonOps.push_back("!=");
-        comparisonOps.push_back("<");
-        comparisonOps.push_back("<=");
-        comparisonOps.push_back(">");
-        comparisonOps.push_back(">=");
-        bitOps.push_back("&");
-        bitOps.push_back("|");
-        bitOps.push_back("^");
-        bitOps.push_back("~");
-
-        extendedOps.push_back(",");
-        extendedOps.push_back("[");
-        extendedOps.push_back("]");
-        extendedOps.push_back("(");
-        extendedOps.push_back(")");
-        extendedOps.push_back("?");
-        extendedOps.push_back(":");
-
-        assignmentOps.push_back("=");
-        assignmentOps.push_back("+=");
-        assignmentOps.push_back("-=");
-        assignmentOps.push_back("*=");
-        assignmentOps.push_back("/=");
-        assignmentOps.push_back("%=");
-        assignmentOps.push_back("&=");
-        assignmentOps.push_back("^=");
-        assignmentOps.push_back("|=");
-        assignmentOps.push_back("<<=");
-        assignmentOps.push_back(">>=");
-    }
-
     void matchOp() {
         std::vector<std::string> test_ops;
         append_vector(test_ops, arithmeticalOps);
@@ -632,7 +795,7 @@ private:
     void isArithmeticalOp() const {
         std::vector<std::string>::const_iterator test_op, test_ops_end = arithmeticalOps.end();
         for (test_op = arithmeticalOps.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isArithmeticalOp());
         }
@@ -647,7 +810,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isArithmeticalOp(), "Failing arithmetical operator: " + *other_op);
         }
@@ -663,7 +826,7 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = test_ops.end();
         for (test_op = test_ops.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isOp());
         }
@@ -674,7 +837,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isOp(), "Failing normal operator: " + *other_op);
         }
@@ -689,7 +852,7 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = test_ops.end();
         for (test_op = test_ops.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isConstOp());
         }
@@ -701,7 +864,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isConstOp(), "Failing normal operator: " + *other_op);
         }
@@ -717,7 +880,7 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = test_ops.end();
         for (test_op = test_ops.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isExtendedOp());
         }
@@ -725,7 +888,7 @@ private:
         // Negative test against assignment operators
         std::vector<std::string>::const_iterator other_op, other_ops_end = assignmentOps.end();
         for (other_op = assignmentOps.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isExtendedOp(), "Failing assignment operator: " + *other_op);
         }
@@ -734,7 +897,7 @@ private:
     void isAssignmentOp() const {
         std::vector<std::string>::const_iterator test_op, test_ops_end = assignmentOps.end();
         for (test_op = assignmentOps.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isAssignmentOp());
         }
@@ -749,7 +912,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isAssignmentOp(), "Failing assignment operator: " + *other_op);
         }
@@ -758,26 +921,26 @@ private:
     void operators() const {
         std::vector<std::string>::const_iterator test_op;
         for (test_op = extendedOps.begin(); test_op != extendedOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eExtendedOp, tok.tokType());
         }
         for (test_op = logicalOps.begin(); test_op != logicalOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eLogicalOp, tok.tokType());
         }
         for (test_op = bitOps.begin(); test_op != bitOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eBitOp, tok.tokType());
         }
         for (test_op = comparisonOps.begin(); test_op != comparisonOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eComparisonOp, tok.tokType());
         }
-        Token tok(nullptr);
+        Token tok;
         tok.str("++");
         ASSERT_EQUALS(Token::eIncDecOp, tok.tokType());
         tok.str("--");
@@ -785,7 +948,7 @@ private:
     }
 
     void literals() const {
-        Token tok(nullptr);
+        Token tok;
 
         tok.str("\"foo\"");
         ASSERT(tok.tokType() == Token::eString);
@@ -805,34 +968,40 @@ private:
 
     void isStandardType() const {
         std::vector<std::string> standard_types;
-        standard_types.push_back("bool");
-        standard_types.push_back("char");
-        standard_types.push_back("short");
-        standard_types.push_back("int");
-        standard_types.push_back("long");
-        standard_types.push_back("float");
-        standard_types.push_back("double");
-        standard_types.push_back("size_t");
+        standard_types.emplace_back("bool");
+        standard_types.emplace_back("char");
+        standard_types.emplace_back("short");
+        standard_types.emplace_back("int");
+        standard_types.emplace_back("long");
+        standard_types.emplace_back("float");
+        standard_types.emplace_back("double");
+        standard_types.emplace_back("size_t");
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = standard_types.end();
         for (test_op = standard_types.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS_MSG(true, tok.isStandardType(), "Failing standard type: " + *test_op);
         }
 
         // Negative test
-        Token tok(nullptr);
+        Token tok;
         tok.str("string");
         ASSERT_EQUALS(false, tok.isStandardType());
 
         // Change back to standard type
         tok.str("int");
         ASSERT_EQUALS(true, tok.isStandardType());
+
+        // token can't be both type and variable
+        tok.str("abc");
+        tok.isStandardType(true);
+        tok.varId(123);
+        ASSERT_EQUALS(false, tok.isStandardType());
     }
 
     void updateProperties() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("foobar");
 
         ASSERT_EQUALS(true, tok.isName());
@@ -844,62 +1013,55 @@ private:
         ASSERT_EQUALS(true, tok.isNumber());
     }
 
-    void updatePropertiesConcatStr() const {
-        Token tok(nullptr);
-        tok.str("true");
-
-        ASSERT_EQUALS(true, tok.isBoolean());
-
-        tok.concatStr("123");
-
-        ASSERT_EQUALS(false, tok.isBoolean());
-        ASSERT_EQUALS("tru23", tok.str());
-    }
-
     void isNameGuarantees1() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("Name");
         ASSERT_EQUALS(true, tok.isName());
     }
 
     void isNameGuarantees2() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("_name");
         ASSERT_EQUALS(true, tok.isName());
     }
 
     void isNameGuarantees3() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("_123");
         ASSERT_EQUALS(true, tok.isName());
     }
 
     void isNameGuarantees4() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("123456");
         ASSERT_EQUALS(false, tok.isName());
         ASSERT_EQUALS(true, tok.isNumber());
     }
 
     void isNameGuarantees5() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("a123456");
         ASSERT_EQUALS(true, tok.isName());
         ASSERT_EQUALS(false, tok.isNumber());
     }
 
+    void isNameGuarantees6() const {
+        Token tok;
+        tok.str("$f");
+        ASSERT_EQUALS(true, tok.isName());
+    }
 
     void canFindMatchingBracketsNeedsOpen() const {
         givenACodeSampleToTokenize var("std::deque<std::set<int> > intsets;");
 
-        const Token* t = var.tokens()->findClosingBracket();
+        const Token* const t = var.tokens()->findClosingBracket();
         ASSERT(t == nullptr);
     }
 
     void canFindMatchingBracketsInnerPair() const {
         givenACodeSampleToTokenize var("std::deque<std::set<int> > intsets;");
 
-        Token* t = const_cast<Token*>(var.tokens()->tokAt(7))->findClosingBracket();
+        const Token * const t = var.tokens()->tokAt(7)->findClosingBracket();
         ASSERT_EQUALS(">", t->str());
         ASSERT(var.tokens()->tokAt(9) == t);
     }
@@ -907,7 +1069,7 @@ private:
     void canFindMatchingBracketsOuterPair() const {
         givenACodeSampleToTokenize var("std::deque<std::set<int> > intsets;");
 
-        const Token* t = var.tokens()->tokAt(3)->findClosingBracket();
+        const Token* const t = var.tokens()->tokAt(3)->findClosingBracket();
         ASSERT_EQUALS(">", t->str());
         ASSERT(var.tokens()->tokAt(10) == t);
     }
@@ -915,7 +1077,7 @@ private:
     void canFindMatchingBracketsWithTooManyClosing() const {
         givenACodeSampleToTokenize var("X< 1>2 > x1;\n");
 
-        const Token* t = var.tokens()->next()->findClosingBracket();
+        const Token* const t = var.tokens()->next()->findClosingBracket();
         ASSERT_EQUALS(">", t->str());
         ASSERT(var.tokens()->tokAt(3) == t);
     }
@@ -929,6 +1091,47 @@ private:
         t = var.tokens()->tokAt(4)->findClosingBracket();
         ASSERT(t == nullptr);
     }
+
+    void findClosingBracket() {
+        givenACodeSampleToTokenize var("template<typename X, typename...Y> struct S : public Fred<Wilma<Y...>> {}");
+
+        const Token* const t = var.tokens()->next()->findClosingBracket();
+        ASSERT(Token::simpleMatch(t, "> struct"));
+    }
+
+    void expressionString() {
+        givenACodeSampleToTokenize var1("void f() { *((unsigned long long *)x) = 0; }");
+        const Token *const tok1 = Token::findsimplematch(var1.tokens(), "*");
+        ASSERT_EQUALS("*((unsigned long long*)x)", tok1->expressionString());
+
+        givenACodeSampleToTokenize var2("typedef unsigned long long u64; void f() { *((u64 *)x) = 0; }");
+        const Token *const tok2 = Token::findsimplematch(var2.tokens(), "*");
+        ASSERT_EQUALS("*((unsigned long long*)x)", tok2->expressionString());
+
+        givenACodeSampleToTokenize data3("void f() { return (t){1,2}; }");
+        ASSERT_EQUALS("return(t){1,2}", data3.tokens()->tokAt(5)->expressionString());
+
+        givenACodeSampleToTokenize data4("void f() { return L\"a\"; }");
+        ASSERT_EQUALS("returnL\"a\"", data4.tokens()->tokAt(5)->expressionString());
+
+        givenACodeSampleToTokenize data5("void f() { return U\"a\"; }");
+        ASSERT_EQUALS("returnU\"a\"", data5.tokens()->tokAt(5)->expressionString());
+    }
+
+    void hasKnownIntValue() {
+        // pointer might be NULL
+        ValueFlow::Value v1(0);
+
+        // pointer points at buffer that is 2 bytes
+        ValueFlow::Value v2(2);
+        v2.valueType = ValueFlow::Value::BUFFER_SIZE;
+        v2.setKnown();
+
+        Token token;
+        ASSERT_EQUALS(true, token.addValue(v1));
+        ASSERT_EQUALS(true, token.addValue(v2));
+        ASSERT_EQUALS(false, token.hasKnownIntValue());
+    }
 };
 
 REGISTER_TEST(TestToken)
